Attribute VB_Name = "AreaLib"
'---------------------------- AreaLib.BAS ----------------------------

'von Gerhard Berg, DJ6FM

DefInt A-Z
'

Sub AddRect(r1 As RECT, r2 As RECT, r3 As RECT)

  r3.Left = r1.Left + r2.Left
  r3.Top = r1.Top + r2.Top
  r3.Right = r1.Right + r2.Right
  r3.Bottom = r1.Bottom + r2.Bottom

End Sub

Function AreaInArea(Mode, tst As lAreaType, ref As lAreaType)

  'Check if area 'tst' is contained in area 'ref'

  'Mode:  0 = tst must be completely in ref
  '       1 = tst must be at least partially in ref
  '       2 = tst and ref must be identical

  AreaInArea = False

  Select Case Mode
  Case 0
    If tst.lxmin < ref.lxmin Then Exit Function
    If tst.lxmax > ref.lxmax Then Exit Function
    If tst.lymin < ref.lymin Then Exit Function
    If tst.lymax > ref.lymax Then Exit Function
  Case 1
    If tst.lxmax < ref.lxmin Then Exit Function
    If tst.lxmin > ref.lxmax Then Exit Function
    If tst.lymax < ref.lymin Then Exit Function
    If tst.lymin > ref.lymax Then Exit Function
  Case 2
    If tst.lxmin <> ref.lxmin Then Exit Function
    If tst.lxmax <> ref.lxmax Then Exit Function
    If tst.lymin <> ref.lymin Then Exit Function
    If tst.lymax <> ref.lymax Then Exit Function
  Case Else
    Stop
  End Select

  AreaInArea = True

End Function

Sub DegArea2SecArea(degArea As degAreaType, secArea As lAreaType, Mode)

  Select Case Mode
  Case 1
    'Area auf naechsten Wert verkleinern
    secArea.lymin = -Int(-degArea.bmin * cfDeg2Sec)
    secArea.lymax = Int(degArea.bmax * cfDeg2Sec)
    secArea.lxmin = -Int(-degArea.lMin * cfDeg2Sec)
    secArea.lxmax = Int(degArea.lMax * cfDeg2Sec)

  Case 2
    'Area auf naechsten Wert vergroessern
    secArea.lymin = Int(degArea.bmin * cfDeg2Sec)
    secArea.lymax = -Int(-degArea.bmax * cfDeg2Sec)
    secArea.lxmin = Int(degArea.lMin * cfDeg2Sec)
    secArea.lxmax = -Int(-degArea.lMax * cfDeg2Sec)

  Case Else
    Stop
  End Select

End Sub

Function degArea2Str$(degArea As degAreaType)

  Const AnzFields = 1
  Const AnzNulls = 2 * (3 - AnzFields) + 3

  With degArea
    m$ = Unformat$(dDegCoord2DegStr$(.lMin, AnzFields, AnzNulls, 0))
    m$ = m$ + ", " + Unformat$(dDegCoord2DegStr$(.bmin, AnzFields, AnzNulls, 0))
    m$ = m$ + ", " + Unformat$(dDegCoord2DegStr$(.lMax, AnzFields, AnzNulls, 0))
    m$ = m$ + ", " + Unformat$(dDegCoord2DegStr$(.bmax, AnzFields, AnzNulls, 0))
  End With

  degArea2Str$ = m$

End Function

Sub ForcePointIntoRect(p As PointApi, r As RECT)

  If p.x < r.Left Then p.x = r.Left
  If p.y < r.Top Then p.y = r.Top
  If p.x >= r.Right Then p.x = r.Right - 1
  If p.y >= r.Bottom Then p.y = r.Bottom - 1

End Sub

Sub ForceRectIntoRect(r1 As RECT, r2 As RECT)

  If r1.Left < r2.Left Then r1.Left = r2.Left
  If r1.Top < r2.Top Then r1.Top = r2.Top
  If r1.Right > r2.Right Then r1.Right = r2.Right
  If r1.Bottom > r2.Bottom Then r1.Bottom = r2.Bottom

End Sub

Sub GetAreaCenter(a As lAreaType, xm&, ym&)

  xm& = (a.lxmax + a.lxmin) \ 2
  ym& = (a.lymax + a.lymin) \ 2

End Sub

Sub GetAreaSize(a As lAreaType, w&, h&)

  w& = a.lxmax - a.lxmin
  h& = a.lymax - a.lymin

End Sub

Sub GetRectCenter(r As RECT, p As PointApi)

  p.x = (r.Left + r.Right - 1) \ 2
  p.y = (r.Top + r.Bottom - 1) \ 2

End Sub

Sub GetRectSize(r As RECT, w&, h&)

  w& = r.Right - r.Left
  h& = r.Bottom - r.Top

End Sub

Sub iArea2lArea(iArea As iAreaType, lArea As lAreaType)

  lArea.lxmin = 20& * iArea.ixmin
  lArea.lxmax = 20& * iArea.ixmax
  lArea.lymin = 10& * iArea.iymin
  lArea.lymax = 10& * iArea.iymax

End Sub

Function IsNullArea(lArea As lAreaType)

  IsNullArea = False

  If lArea.lxmin <> 0 Then Exit Function
  If lArea.lymin <> 0 Then Exit Function
  If lArea.lxmax <> 0 Then Exit Function
  If lArea.lymax <> 0 Then Exit Function

  IsNullArea = True

End Function

Function IsWorldArea(lArea As lAreaType)

  IsWorldArea = False

  If lArea.lxmin <> -clMaxDegX Then Exit Function
  If lArea.lymin <> -clMaxDegY Then Exit Function
  If lArea.lxmax <> clMaxDegX Then Exit Function
  If lArea.lymax <> clMaxDegY Then Exit Function

  IsWorldArea = True

End Function

Sub lArea2iArea(lArea As lAreaType, iArea As iAreaType, Mode)

  Select Case Mode
  Case 1
    'Area auf naechsten Wert verkleinern
    iArea.ixmin = Int((lArea.lxmin + 19) / 20)
    iArea.ixmax = Int(lArea.lxmax / 20)
    iArea.iymin = Int((lArea.lymin + 9) / 10)
    iArea.iymax = Int(lArea.lymax / 10)
  Case 2
    'Area auf naechsten Wert vergroessern
    iArea.ixmin = Int(lArea.lxmin / 20)
    iArea.ixmax = Int((lArea.lxmax + 19) / 20)
    iArea.iymin = Int(lArea.lymin / 10)
    iArea.iymax = Int((lArea.lymax + 9) / 10)
  Case Else
    Stop
  End Select

End Sub

Function lArea2Msg$(lArea As lAreaType, AnzFields)

  s1$ = " ... "
  s2$ = "     "

  m$ = lSecCoord2Msg$(lArea.lymin, AnzFields, 1)
  m$ = m$ & s1$ & lSecCoord2Msg$(lArea.lymax, AnzFields, 1)
  m$ = m$ & s2$ & lSecCoord2Msg$(lArea.lxmin, AnzFields, 2)
  m$ = m$ & s1$ & lSecCoord2Msg$(lArea.lxmax, AnzFields, 2)

  lArea2Msg$ = m$

End Function

Sub lArea2QthArea(lArea As lAreaType, QthArea As iAreaType)

  QthArea.ixmin = lArea.lxmin * cfSec2Qth
  QthArea.ixmax = lArea.lxmax * cfSec2Qth
  QthArea.iymin = lArea.lymin * cfSec2Qth
  QthArea.iymax = lArea.lymax * cfSec2Qth

End Sub

Function lArea2Str$(lArea As lAreaType)

  m$ = Unformat$(lSecCoord2StrEx$(lArea.lxmin, 1))
  m$ = m$ + ", " + Unformat$(lSecCoord2StrEx$(lArea.lymin, 1))
  m$ = m$ + ", " + Unformat$(lSecCoord2StrEx$(lArea.lxmax, 1))
  m$ = m$ + ", " + Unformat$(lSecCoord2StrEx$(lArea.lymax, 1))

  lArea2Str$ = m$

End Function

Function PointInIArea(x, y, a As iAreaType)

  'Check if point is in area

  PointInIArea = False

  If x < a.ixmin Then Exit Function
  If x > a.ixmax Then Exit Function
  If y < a.iymin Then Exit Function
  If y > a.iymax Then Exit Function

  PointInIArea = True

End Function

Function PointInLArea(x&, y&, a As lAreaType)

  'Check if point is in area

  PointInLArea = False

    If x& < a.lxmin Then Exit Function
    If x& > a.lxmax Then Exit Function
    If y& < a.lymin Then Exit Function
    If y& > a.lymax Then Exit Function

  PointInLArea = True

End Function

Function PointInRect(p As PointApi, r As RECT)

  PointInRect = False

  If p.x < r.Left Then Exit Function
  If p.x >= r.Right Then Exit Function
  If p.y < r.Top Then Exit Function
  If p.y >= r.Bottom Then Exit Function

  PointInRect = True

End Function

Function PointInWorld(x&, y&)

  'Check if point is in world

  PointInWorld = False

  If x& < -clMaxDegX Then Exit Function
  If x& > clMaxDegX Then Exit Function
  If y& < -clMaxDegY Then Exit Function
  If y& > clMaxDegY Then Exit Function

  PointInWorld = True

End Function

Function RectInRect(Mode, tst As RECT, ref As RECT)

  'Check if area 'tst' is contained in area 'ref'

  'Mode:  0 = tst must be completely in ref
  '       1 = tst must be at least partially in ref
  '       2 = tst and ref must be identical

  RectInRect = False

  Select Case Mode
  Case 0
    If tst.Left < ref.Left Then Exit Function
    If tst.Top < ref.Top Then Exit Function
    If tst.Right > ref.Right Then Exit Function
    If tst.Bottom > ref.Bottom Then Exit Function

  Case 1
    If tst.Right < ref.Left Then Exit Function
    If tst.Bottom < ref.Top Then Exit Function
    If tst.Left > ref.Right Then Exit Function
    If tst.Top > ref.Bottom Then Exit Function

  Case 2
    If tst.Left <> ref.Left Then Exit Function
    If tst.Top <> ref.Top Then Exit Function
    If tst.Right <> ref.Right Then Exit Function
    If tst.Bottom <> ref.Bottom Then Exit Function

  End Select

  RectInRect = True

End Function

Sub secArea2degArea(secArea As lAreaType, degArea As degAreaType)

  degArea.bmin = secArea.lymin * cfSec2Deg
  degArea.bmax = secArea.lymax * cfSec2Deg
  degArea.lMin = secArea.lxmin * cfSec2Deg
  degArea.lMax = secArea.lxmax * cfSec2Deg

End Sub

Sub SetArea(xm&, ym&, w&, h&, a As lAreaType)

  a.lxmin = xm& - w& / 2
  a.lxmax = xm& + w& / 2
  a.lymin = ym& - h& / 2
  a.lymax = ym& + h& / 2

End Sub

Sub SetWorldArea(lArea As lAreaType)

  'Max. Kartenbereich
  lArea.lxmin = -clMaxDegX
  lArea.lxmax = clMaxDegX
  lArea.lymin = -clMaxDegY
  lArea.lymax = clMaxDegY

End Sub

Function Str2degArea(ByVal m$, degArea As degAreaType)

  With degArea
    On Error Resume Next
    .lMin = Val(Parse$(m$, ","))
    .bmin = Val(Parse$(m$, ","))
    .lMax = Val(Parse$(m$, ","))
    .bmax = Val(Parse$(m$, ","))
    Str2degArea = Err
    On Error GoTo 0
  End With

End Function

Function Str2lArea(ByVal m$, lArea As lAreaType)

  On Error Resume Next
  lArea.lxmin = Val(Parse$(m$, ",")) * cfDeg2Sec
  lArea.lymin = Val(Parse$(m$, ",")) * cfDeg2Sec
  lArea.lxmax = Val(Parse$(m$, ",")) * cfDeg2Sec
  lArea.lymax = Val(Parse$(m$, ",")) * cfDeg2Sec
  Str2lArea = Err
  On Error GoTo 0

End Function

Function Str2Rect(m$, r As RECT)

  On Error Resume Next
  r.Top = Val(Parse$(m$, ","))
  r.Bottom = Val(Parse$(m$, ","))
  r.Left = Val(Parse$(m$, ","))
  r.Right = Val(Parse$(m$, ","))
  Str2Rect = Err
  On Error GoTo 0

End Function

Sub SubArea(a1 As lAreaType, a2 As lAreaType, a3 As lAreaType)

  a3.lxmin = a1.lxmin - a2.lxmin
  a3.lxmax = a1.lxmax - a2.lxmax
  a3.lymin = a1.lymin - a2.lymin
  a3.lymax = a1.lymax - a2.lymax

End Sub

Sub SubRect(r1 As RECT, r2 As RECT, r3 As RECT)

  r3.Left = r1.Left - r2.Left
  r3.Top = r1.Top - r2.Top
  r3.Right = r1.Right - r2.Right
  r3.Bottom = r1.Bottom - r2.Bottom

End Sub

