Attribute VB_Name = "BL_EN"
'----------------------------------------------------------------------'
'  Programme BL <-> EN                                       bl-en.for '
'                                                                      '
'  http://habicht.bauv.unibw-muenchen.de/scherer/blyx.html             '
'                                                                      '
'  Autor:   Dipl.-Ing. Bernd Scherer                                   '
'  ------   Universitaet der Bundeswehr Muenchen                       '
'           D-85577 Neubiberg                                          '
'                                                                      '
'  E-mail:  a91alvub@unibw-muenchen.de                      28.10.1998 '
'  -------  Bernd.Scherer@t-online.de                                  '
'----------------------------------------------------------------------'
'  Portierung von FORTRAN nach Visual Basic                  bl-en.bas '
'  ----------------------------------------                            '
'           Gerhard Berg, DJ6FM                                        '
'  E-mail:  HAMMAP@T-Online.de                              17.05.1999 '
'----------------------------------------------------------------------'
'
'  Literatur:
'  ----------
'  - Autor: Heitz, Siegfried
'  - Titel: Koordinaten auf geodaetischen Bezugsflaechen
'  - Verlag: Duemmler Verlag, Bonn 1985, Duemmlerbuch 7898
'  - ISBN 3-427-78981-0


'  zum Hauptprogramm
'  -----------------
'  (mit default-Werten fuer e0,b0#)
      'e0# = 500000#
      'b0# = 0#
      'Call blen(n, e, l#, b#, c#, E2#, Pi#, L0#, e0#, b0#, m#)
      'Call enbl(n, e, l#, b#, c#, E2#, Pi#, L0#, e0#, b0#, m#)


'     Vereinbarungen
'     --------------
      DefInt I-J
      DefDbl A-H, K-Z

'
'


'  -----------------------------
'  Unterprogramme fuer BL <-> EN
'  -----------------------------
'  +------------------------ +
'  | 1.                      |
'  | Transformation EN -> BL |
'  +-------------------------+

      Sub bl2en(x#, y#, l#, b#, c#, E2#, Pi#, L0#, e0#, b0#, m#)
'
'          *************************************************
'          *            TRANSFORMATIONSPROGRAMM            *
'          * ELLIPSOIDISCHE GEOGRAPHISCHE KOORDINATEN  L,B *
'          *       IN GAUSS-KRUEGER-KOORDINATEN  X,Y       *
'          *           ODER UTM-KOORDINATEN  E,N           *
'          *************************************************
'
'
'    Variablen: Bezeichnung wie oben
'
'    Die Berechnungen erfolgen nach HEITZ: "Koordinaten auf geodaeti-
'    schen Bezugsflaechen", Kapitel 4
'
'
'    0. Vereinbarungen
'    -----------------
     Static a(2, 5), U(3)
     Static KA(5), K1(4), K2(3), K3(2)
'
'
'    1. Bezugsmeridian
'    -----------------
'
'    1.1 Differenz der Laenge zum Bezugsmeridian als Argument des
'        Reihenansatzes fuer die konformen Koordinaten
'
      DL = l# - L0#
'*****************************
' Abfrage zur Regelung des Null-Meridian-Uebergangs
      If (DL > 0.7) Then DL = DL - 2# * Pi#
      If (DL < -0.7) Then DL = DL + 2# * Pi#
'*****************************
'
'
'    3. Berechnung Bogenlaenge entlang Mittelmeridian
'    ------------------------------------------------
'
'    Gleichungen (4-50a) bis (4-50d), Seite 127
'
'
'    3.1 Vorbereitung
'
      Call LBINXYK(KA(), K1(), K2(), K3())
      L3 = 0
      BB = Abs(b#)
      If (BB <> 0#) Then L3 = b# / BB
'
'
'    3.2 Argumente der Reihenentwicklung
'
      f2 = E2# * E2#
      F4 = f2 * f2
      F6 = F4 * f2
      F8 = F4 * F4
      F10 = F4 * F6
'
'
'    3.3 Koeffizienten
'
      A0 = c# * (1# + KA(1) * f2 + KA(2) * F4 + KA(3) * F6 + KA(4) * F8 + KA(5) * F10)
      U(1) = c# * (K1(1) * f2 + K1(2) * F4 + K1(3) * F6 + K1(4) * F8)
      U(2) = c# * (K2(1) * F4 + K2(2) * F6 + K2(3) * F8)
      U(3) = c# * (K3(1) * F6 + K3(2) * F8)
'
'
'    3.4 Bogenlaenge entlang dem Mittelmeridian zwischen dem Aequator
'        (Breite = b0) und der aktuellen Breite B
'
      BB = BB - b0#
      s = A0 * BB
      For i = 1 To 3
        s = s + U(i) * Sin(CDbl(i) * BB * 2#)
      Next i
'
'
'    4. Reihenansatz
'    ---------------
'
'    4.1 Vorbereitung der Variablen
'
      co = Cos(BB)
      c2 = co * co
      C4 = c2 * c2
      C5 = C4 * co
      T1 = Tan(BB - 0.000001)
      If (BB < Pi# / 2#) Then T1 = Tan(BB)
      T2 = T1 * T1
      T4 = T2 * T2
      V2 = 1# + f2 * c2
      r1 = c# / Sqr(V2)
'
'
'    4.2 Koeffizienten der Reihenentwicklung
'
'    Gleichungen (4-93g), Seite 148
'
      For i = 1 To 2
        U(i) = 0#
        For j = 1 To 5
          a(i, j) = 0#
        Next j
      Next i

      a(2, 1) = r1 * co
      a(1, 2) = a(2, 1) * co * T1 / 2#
      a(2, 3) = a(2, 1) * c2 * (V2 - T2) / 6#
      a(1, 4) = -r1 * C4 * T1 * (4# - 9# * V2 + T2) / 24#
      a(2, 5) = r1 * C5 * (5# - 18# * T2 + T4) / 120#
'
'
'    4.3 Reihe in dL
'
'    Gleichung (4-95b), Seite 149
'
      For i = 1 To 2
        For j = 1 To 5
          U(i) = U(i) + a(i, j) * DL ^ j
        Next j
      Next i
'
'
'    5. Konforme Koordinaten x,y
'    --------------------------------
      x# = (U(1) + s) * L3 * m#
      y# = U(2) * m# + e0#
'
'
'    6.  Programmende
'    ----------------

      End Sub

      Sub en2bl(x#, y#, l#, b#, c#, E2#, Pi#, L0#, e0#, b0#, m#)

'
'          *************************************************
'          *            TRANSFORMATIONSPROGRAMM            *
'          *         GAUSS-KRUEGER-KOORDINATEN  H,R        *
'          *        ODER UTM-KOORDINATEN  E,N     IN       *
'          * ELLIPSOIDISCHE GEOGRAPHISCHE KOORDINATEN  L,B *
'          *************************************************
'
'    Variablen:  x ...... GAUSS-KRUEGER-Koordinate H, UTM-Koordinate N
'                y ...... GAUSS-KRUEGER-Koordinate R, UTM-Koordinate E
'                l ...... geographische Laenge L (in Radiant)
'                b ...... geographische Breite B (in Radiant)
'                c ...... Ellipsoidparameter c (Polkruemmungshalbmesser)
'                     WGS84:  6399593.626
'                     BESSEL: 6398786.849
'                     INT:    6399936.608 (ED50, UTM)
'                e2 ..... Ellipsoidparameter e' (2. numerische Exzentr.)
'                     WGS84:  0.082094438
'                     BESSEL: 0.081970843
'                     INT:    0.082268890 (ED50, UTM)
'                pi ..... Zahl pi = 4 * arctan 1
'                l0 ..... Laenge des Bezugsmeridians (in Radiant)
'                e0 ..... false easting (default = 500000.000m)
'                b0 ..... false northing (default = 0.000000rad)
'                m ...... = 1 fuer GAUSS-KRUEGER, = 0.9996 fuer UTM
'
'    Die Berechnungen erfolgen nach HEITZ: "Koordinaten auf geodaeti-
'    schen Bezugsflaechen", Kapitel 4
'
'
'    0. Vereinbarungen
'    -----------------
     Static a(2, 5), U(3)
     Static KA(5), K1(4), K2(3), K3(2)
'
'
'    1. Berechnung der Breite des Fusspunktes am Bezugsmeridian
'    ----------------------------------------------------------
'
'    Gleichungen (4-51), Seite 128
'
'
'    1.1 Argumente der Reihenentwicklung
'
      f2 = E2# * E2#
      F4 = f2 * f2
      F6 = F4 * f2
      F8 = F4 * F4
      F10 = F6 * F4
'
'
'    1.2 Koeffizienten
'
      Call XYINLBK(KA(), K1(), K2(), K3())
      A0 = c# * (1# + KA(1) * f2 + KA(2) * F4 + KA(3) * F6 + KA(4) * F8 + KA(5) * F10)
      U(1) = K1(1) * f2 + K1(2) * F4 + K1(3) * F6 + K1(4) * F8
      U(2) = K2(1) * F4 + K2(2) * F6 + K2(3) * F8
      U(3) = K3(1) * F6 + K3(2) * F8
'
'
'    1.3 Breite des Fusspunktes
'
      BQ = Abs(x# / m#) / A0
      BF = BQ
      For i = 1 To 3
        BF = BF + U(i) * Sin(CDbl(i) * BQ * 2#)
      Next i
'
'
'    3. Reihenansatz
'    ---------------
'
'    3.1 Vorbereitung der Variablen
'
      co = Cos(BF)
      T1 = Tan(BF - 0.000001)
      If (BF < Pi# / 2#) Then T1 = Tan(BF)
      T2 = T1 * T1
      T4 = T2 * T2
      V2 = 1# + f2 * co * co
      V4 = V2 * V2
      r1 = Sqr(V2) / c#
      r3 = r1 * r1 * r1
      R4 = r3 * r1
      R5 = R4 * r1
'
'
'    3.2 Koeffizienten
'
'    Gleichungen (4-95c) und (4-96b), Seite 149/150
'
      For i = 1 To 2
        For j = 1 To 5
          a(i, j) = 0#
        Next j
      Next i

      a(2, 1) = r1 * co * (1# + T2)
      a(1, 2) = -V4 / c# / c# * T1 / 2#
      a(2, 3) = -r3 * co * (1# + T2) * (V2 + T2 + T2) / 6#
      a(1, 4) = -R4 * T1 * (1# - 6# * V2 - 3# * (3# - V2 - V2) * T2) / 24#
      a(2, 5) = R5 * co * (1# + T2) * (5# + 28# * T2 + 24# * T4) / 120#
'
'
'    3.3 Argument der Reihe:  dy=y-500000
'
      dy = y# - e0#
      dy = dy / m#
'
'
'    3.4 Reihe in dy
'
      For i = 1 To 2
        U(i) = 0#
        For j = 1 To 5
          U(i) = U(i) + a(i, j) * dy ^ j
        Next j
      Next i
'
'
'    4. Geographische Koordinaten L,B
'    --------------------------------
      b# = U(1) + BF + b0#
      l# = U(2) + L0#
      If (l# < 0#) Then l# = l# + Pi# + Pi#
      If (x# < 0#) Then b# = -b#
'
'
'    5. Programmende
'    ---------------
      End Sub
'
'
'

      Sub LBINXYK(Q1(), Q2(), Q3(), Q4())
'
'          *************************************************
'          *            KONSTANTEN FUER PROGRAMM           *
'          *                    LBINXY                     *
'          *     ( BERECHNUNG DER MERIDIANBOGENLAENGE )    *
'          *************************************************
'
'
'    1. Konstantensatz 1
'    -------------------
      Q1(1) = -0.75
      Q1(2) = 45# / 64#
      Q1(3) = -175# / 256#
      Q1(4) = 11025# / 16384#
      Q1(5) = -10395# / 32768#
'
'
'    2. Konstantensatz 2
'    -------------------
      Q2(1) = -0.375
      Q2(2) = 15# / 32#
      Q2(3) = -525# / 1024#
      Q2(4) = 2205# / 4096#
'
'
'    3. Konstantensatz 3
'    -------------------
      Q3(1) = 15# / 256#
      Q3(2) = -105# / 1024#
      Q3(3) = 2205# / 16384#
'
'
'    4. Konstantensatz 4
'    -------------------
      Q4(1) = -35# / 3072#
      Q4(2) = 315# / 12288#
'
'
'    5. Ende der Definition
'    ----------------------
      End Sub

      Sub XYINLBK(Q1(), Q2(), Q3(), Q4())

'
'          *************************************************
'          *            KONSTANTEN FUER PROGRAMM           *
'          *                    XYINLB                     *
'          *   ( BERECHNUNG DER BREITE DES FUSSPUNKTES )   *
'          *************************************************
'
'
'    1. Konstantensatz 1
'    -------------------
      Q1(1) = -0.75
      Q1(2) = 45# / 64#
      Q1(3) = -175# / 256#
      Q1(4) = 11025# / 16384#
      Q1(5) = -10395# / 32768#
'
'
'    2. Konstantensatz 2
'    -------------------
      Q2(1) = 0.375
      Q2(2) = -3# / 16#
      Q2(3) = 213# / 2048#
      Q2(4) = -255# / 4096#
'
'
'    3. Konstantensatz 3
'    -------------------
      Q3(1) = 21# / 256#
      Q3(2) = -Q3(1)
      Q3(3) = 533# / 8192#
'
'
'    4. Konstantensatz 4
'    -------------------
      Q4(1) = 151# / 6144#
      Q4(2) = -453# / 12288#
'
'
'    5. Ende der Definition
'    ----------------------
      End Sub



'  +------------------------ +
'  | 2.                      |
'  | Transformation BL -> EN |
'  +-------------------------+

