Attribute VB_Name = "BL_EN_IF"
DefInt A-Z

Function CoordTransf(FromType, FromDeg As dGeoPoint, ToType, ToDeg As dGeoPoint)

  'FromDeg und ToDeg = Koordinaten in Grad
  'FromType, ToType: 0=WGS84, 1=ED50, 2=Potsdam, 3=CH1903

  Dim FromDat As GeoDatum
  Dim ToDat As GeoDatum

  Dim FromRad As dGeoPoint
  Dim ToRad As dGeoPoint

  CoordTransf = True

  If ToType = FromType Then

    ToDeg = FromDeg

  Else
    void = GetDatum(FromType, FromDat)
    void = GetDatum(ToType, ToDat)

    FromRad.b = FromDeg.b * deg2rad
    FromRad.l = FromDeg.l * deg2rad
    FromRad.h = FromDeg.h

    Call DatumTransf(FromDat, FromRad, ToDat, ToRad)

    ToDeg.b = ToRad.b * rad2deg
    ToDeg.l = ToRad.l * rad2deg
    ToDeg.h = ToRad.h

  End If

  If Abs(ToDeg.b) > 90 Then Exit Function
  If Abs(ToDeg.l) > 180 Then Exit Function

  CoordTransf = False

End Function

Sub DatumTransf(FromDat As GeoDatum, FromCoord As dGeoPoint, ToDat As GeoDatum, ToCoord As dGeoPoint)

  'Lnge und Breite in Rad, Hhe und Radien in m

  Dim CB As Double

  dx# = FromDat.dx - ToDat.dx
  dy# = FromDat.dy - ToDat.dy
  dZ# = FromDat.dZ - ToDat.dZ

  sb# = Sin(FromCoord.b)
  CB# = Cos(FromCoord.b)
  sl# = Sin(FromCoord.l)
  cl# = Cos(FromCoord.l)

  'Delta values
  DB# = -dx# * sb# * cl# - dy# * sb# * sl# + dZ# * CB#
  DL# = (-dx# * sl# + dy# * cl#) / CB#
  dh# = dx# * CB# * cl# + dy# * CB# * sl# + dZ# * sb#


  'Standard Molodensky Datum Transformation
  'Quelle: http://www.utexas.edu/depts/grg/gcraft/notes/datum/gif/molodens.gif

  'Abridged (abgekuerzte) Molodensky Datum Transformation
  'Quelle: http://164.214.2.59/GandG/tm83581/tr83581b.htm

  da# = ToDat.a - FromDat.a       'Delta equitorial radius
  df# = ToDat.f - FromDat.f       'Delta flattening


  from_es# = FromDat.e1sq            '1. oder 2. Num. Exzentr. ??
  sbsq# = sb# * sb#
  nn# = 1# - from_es# * sbsq#

  'Radius of curvature in prime vertical
  Rn# = FromDat.a / Sqr(nn#)

  'Radius of curvature in prime meridian
  Rm# = FromDat.a * (1 - from_es#) / (nn# ^ 1.5)

  'Standard Molodensky

  'Polar radius divided by equatorial radius   bda = 1-f = b/a
  bda# = 1# - FromDat.f

  DB# = DB# + da# * (Rn# * from_es# * sb# * CB#) / FromDat.a
  DB# = DB# + df# * (Rm# / bda# + Rn# * bda#) * sb# * CB#
  DB# = DB# / (Rm# + FromCoord.h)

  DL# = DL# / (Rn# + FromCoord.h)

  dh# = dh# - da# * FromDat.a / Rn# + df# * bda# * Rn# * sbsq#



  ToCoord.b = FromCoord.b + DB#
  ToCoord.l = FromCoord.l + DL#
  ToCoord.h = FromCoord.h + dh#

End Sub

Function dCoord2MGRS(MapDatum, Geo As dGeoPoint, MGRS$)

  'MGRS = Military Grid Reference System (UTM)

  Dim Dat As GeoDatum
  Dim Proj As MapProjection

  Const UtmChr = "ABCDEFGHJKLMNPQRSTUVWXYZ"

  dCoord2MGRS = True

  MGRS$ = ""
  If Geo.b < -80 Or Geo.b > 84 Then Exit Function     'Definitionsbereich
  If Geo.l < -180 Or Geo.l > 180 Then Exit Function

  l# = Geo.l * deg2rad            'geographische Laenge L
  b# = Geo.b * deg2rad            'geographische Breite B


  'Zone          Bezugs-       Streifen
  '              meridian
  '--------------------------------------
  '   1           -177         -180 ... -174
  '  31              3            0 ...    6
  '  32              9            6 ...   12

  Zone# = Int((Geo.l + 180) / 6#) + 1

  'Lnge des Bezugsmeridian
  L0# = ((Zone# - 1) * 6 - 180 + 3) * deg2rad

  void = GetDatum(MapDatum, Dat)
  Call GetProjection(1, Proj)     'UTM

  Call bl2en(n#, e#, l#, b#, Dat.c, Dat.E2, Pi, L0#, Proj.e0, Proj.b0, Proj.m)

  Rechts# = e# + Zone# * 1000000#
  If n# < 0 Then n# = n# + 10000000#           'False Northing
  Hoch# = n#


  '8 Breitenbereich
  'Band       Bereich
  '-----------------------
  '  C        -80 ... -72

  Band$ = Mid$(UtmChr, Int((Geo.b + 80) / 8) + 3, 1)
  If Band$ = "Y" Then Band$ = "X"         'Band X ist 12 breit


  '100 km Quadrate - 1. Buchstabe = Lnge
  '100 km breit
  Q1 = Int(e# / 100000#)
  'je Zone um 8 Buchstaben versetzt - Wiederholung nach je 18 Zonen
  Q2 = ((Zone# - 1) Mod 3) * 8
  qx$ = Mid$(UtmChr, Q1 + Q2, 1)


  '100 km Quadrate - 2. Buchstabe = Breite
  '100 km hoch - Wiederholung alle 2000 km = alle 20 Quadrate
  Q1 = Int(n# / 100000#) Mod 20
  'ungerade Zonen: A...V, gerade Zonen: F...V,A...E
  Q2 = ((Zone# - 1) Mod 2) * 5
  q = (Q1 + Q2) Mod 20 + 1
  qy$ = Mid$(UtmChr, q, 1)


  MGRS$ = Format$(Zone#) + Band$ + " " + qx$ + qy$
  MGRS$ = MGRS$ + " " + Format$(Val(Right$(Str$(e1), 5)), "00000")
  MGRS$ = MGRS$ + " " + Format$(Val(Right$(Str$(Int(n#)), 5)), "00000")

  dCoord2MGRS = False

End Function

Function dCoord2PosStr(OutMapDatum, MapCoSys, DegMinSec, ExtraNulls, pDegIn As dGeoPoint, mb$, ml$)

  Dim pDeg As dGeoPoint, Konf As KonfCoord

  dCoord2PosStr = True
  mb$ = ""
  ml$ = ""

  'Input always in WGS84
  ErrNr = CoordTransf(0, pDegIn, OutMapDatum, pDeg)
  If ErrNr Then Exit Function

  If MapCoSys = 0 Then
    'Geogr.Koord. (Grad)
    AnzFields = DegMinSec + 1
    AnzNulls = 2 * (3 - AnzFields) + ExtraNulls
    mb$ = dDegCoord2DegStr$(pDeg.b, AnzFields, AnzNulls, 1)
    ml$ = dDegCoord2DegStr$(pDeg.l, AnzFields, AnzNulls, 2)

  Else
    ErrNr = Geo2Konf(OutMapDatum, MapCoSys, pDeg, Konf)
    If ErrNr Then Exit Function

    Fmt$ = "#,##0"

    If MapCoSys = 1 Then
      'UTM
      mb$ = Mid$(Msg$(40200), 1 + 2 * Konf.South, 1) & " " & Format$(Konf.Up, Fmt$)
      ml$ = Format$(Konf.Zone, "00") & "  " & Format$(Konf.Right, Fmt$)
    Else
      'Gau-Krger, Schweizer Gitter
      HR$ = Msg$(40201)
      mb$ = Mid$(HR$, 1, 1) & " " & Format$(Konf.Up, Fmt$)
      ml$ = Mid$(HR$, 2, 1) & " " & Format$(Konf.Right + Konf.Zone * 1000000#, Fmt$)
    End If
  End If

  dCoord2PosStr = False

End Function

Function DefaultDatum(MapCoSys)

  If MapCoSys < 2 Then
    'Grad -> WGS84, UTM -> WGS84
    DefaultDatum = 0
  Else
    'Gau-Krger -> Potsdam, Schweizer Gitter -> CH1903
    DefaultDatum = MapCoSys
  End If

End Function

Function Geo2Konf(ByVal MapDatum, ByVal MapCoSys, Geo As dGeoPoint, Konf As KonfCoord)

  'Umwandlung von geografischen in konforme Koordinaten

  'MapDatum: 0=WGS84, 1=ED50, 2=Potsdam, 3=CH1903
  'MapCoSys: 0=Grad,  1=UTM,  2=Gau-Krger, 3=Schweizer-Gitter,
  '         -1=default konf. Koord. je nach Mapdatum

  Dim Dat As GeoDatum
  Dim Proj As MapProjection
  Dim Phi As Double

  Geo2Konf = True

  Konf.h = Geo.h
  Konf.hNN = Geo.hNN

  void = GetDatum(MapDatum, Dat)

  If MapCoSys = 3 Then

    '----- Schweizer Gitter - Nherungsformel -----
    'Quelle: Bundesamt fr Landestopographie - Eidgenssische Vermessungsdirektion
    '  "Formeln und Konstanten fr die Berechnung der Schweizerischen schiefachsigen
    '  Zylinderprojektion und der Transformation zwischen Koordinatensystemen - Januar 2001"
    '  ftp.swisstopo.ch/pub/data/geo/refsysd.pdf
    'siehe Abschnitt 3.5

    If Geo.l < 0 Or Geo.b < 0 Then Exit Function

    b# = Geo.b * 3600 / 10000#       'geographische Breite B in 10000 "
    l# = Geo.l * 3600 / 10000#       'geographische Laenge L in 10000 "

    Phi = b# - 16.902866
    Lambda# = l# - 2.67825
    LambdaSq# = Lambda# * Lambda#

    y1# = 0.2114285339 + Phi * (-0.010939608 + Phi * (-0.000002658 - Phi * 0.00000853))
    y3# = -0.0000442327 + Phi * (0.000004291 - Phi * 0.000000309)
    y5# = 0.0000000197
    BigY# = Lambda# * (y1# + LambdaSq# * (y3# + LambdaSq# * y5#))

    x0# = Phi * (0.3087707463 + Phi * (0.000075028 + Phi * (0.000120435 + Phi * Phi * 0.00000007)))
    x2# = 0.0037454089 + Phi * (-0.0001937927 + Phi * (0.00000434 - Phi * 0.000000376))
    x4# = -0.0000007346 + Phi * 0.0000001444
    BigX# = x0# + LambdaSq# * (x2# + LambdaSq# * x4#)

    Konf.Zone = 0
    Konf.Right = (BigY# + 0.6) * 1000000       'in m
    Konf.Up = (BigX# + 0.2) * 1000000

  Else

    'UTM / Gau-Krger
    Call GetProjection(MapCoSys, Proj)

    l# = Geo.l * deg2rad         'geographische Laenge L
    b# = Geo.b * deg2rad         'geographische Breite B


    Select Case MapCoSys
    Case 1                'UTM
      'Zone          Bezugs-       Streifen
      '              meridian
      '--------------------------------------
      '   1           -177         -180 ... -174
      '  31              3            0 ...    6
      '  32              9            6 ...   12

      If Geo.b < -80 Or Geo.b > 84 Then Exit Function     'Definitionsbereich
      Zone# = Int((Geo.l + 180) / 6#) + 1
      L0# = ((Zone# - 1) * 6 - 180 + 3) * deg2rad         'Bezugsmeridian


    Case 2                'Gau-Krger (Bessel)

      'Zone          Bezugs-       Streifen
      '(Kennziffer)  meridian
      '--------------------------------------
      '   0             0          -1,5 ... 1,5
      '   1             3           1,5 ... 4,5

      If Geo.l < 0 Or Geo.b < 0 Then Exit Function
      Zone# = CInt(Geo.l / 3#)
      L0# = Zone# * 3 * deg2rad      'Laenge des Bezugsmeridians

    End Select

  '    Variablen:  x ...... GAUSS-KRUEGER-Koordinate H, UTM-Koordinate N
  '                y ...... GAUSS-KRUEGER-Koordinate R, UTM-Koordinate E
  '                l ...... geographische Laenge L (in Radiant)
  '                b ...... geographische Breite B (in Radiant)
  '                c ...... Ellipsoidparameter c (Polkruemmungshalbmesser)
  '                     WGS84:  6399593.626
  '                     BESSEL: 6398786.849
  '                     INT:    6399936.608 (ED50, UTM)
  '                e2 ..... Ellipsoidparameter e' (2. numerische Exzentr.)
  '                     WGS84:  0.082094438
  '                     BESSEL: 0.081970843
  '                     INT:    0.082268890 (ED50, UTM)
  '                pi ..... Zahl pi = 4 * arctan 1
  '                l0 ..... Laenge des Bezugsmeridians (in Radiant)
  '                e0 ..... false easting (default = 500000.000m)
  '                b0 ..... false northing (default = 0.000000rad)
  '                m ...... = 1 fuer GAUSS-KRUEGER, = 0.9996 fuer UTM
  '
    Call bl2en(n#, e#, l#, b#, Dat.c, Dat.E2, Pi, L0#, Proj.e0, Proj.b0, Proj.m)

    Konf.Right = e#     '+ Zone# * 1000000#
    Konf.Up = n#
    Konf.Zone = Zone#

    If Geo.b >= 0 Then
      Konf.South = 0
    Else
      Konf.South = 1
    End If

    If Konf.South Then
      'UTM false Northing for southern hemisphere
      Konf.Up = Konf.Up + 10000000#
    End If

  End If

  Geo2Konf = False

End Function

Function GetDatum(Mode, Datum As GeoDatum)

  'Quelle fuer Ellipsen:
  '   http://home.t-online.de/home/Berd.Scherer/ellips.htm
  'Quelle fuer Datumsuebergaenge:
  '  http://home.t-online.de/home/Berd.Scherer/gdtp.htm

  'Quelle fr Schweiz: Bundesamt fr Landestopographie - Eidgenssische Vermessungsdirektion
  '  "Das Schweizer Referenzsystem und die Kartenprojektion"
  '  www.swisstopo.ch/de/geo/refsys.htm

  GetDatum = False

  With Datum

    Select Case Mode
    Case 0       'WGS 84 - WGS 84 Ellipsoid
      'Ellipse
      .a = 6378137                 'Grosse Halbachse
      .b = 6356752.314245          'Kleine Halbachse

      'Shift
      .dx = 0#                     'Differenz in m
      .dy = 0#
      .dZ = 0#


    Case 1     'ED 50 - International 1924 Ellipsoid (= Hayford 1909)
      'MEAN FOR Austria; Belgium; Denmark; Finland; France; W Germany;
      '  Gibraltar; Greece; Italy; Luxembourg; Netherlands; Norway;
      '  Portugal; Spain; Sweden; Switzerland
      'Ellipse
      .a = 6378388
      .b = 6356911.946

      'Shift
      .dx = -87#                   'Datumsuebergang EDS50 -> WGS84
      .dy = -98#
      .dZ = -121#


    Case 2      'Potsdam Datum - Bessel 1841 Ellipsoid
      .a = 6377397.155
      .b = 6356078.962822

      .dx = 606#                   'Datumsuebergang PD -> WGS84
      .dy = 23#
      .dZ = 413#


    Case 3      'CH 1903 - Bessel 1841 Ellipsoid
      .a = 6377397.155
      .b = 6356078.962822

      .dx = 674.374                    'Datumsuebergang CH 1903 -> WGS84
      .dy = 15.056
      .dZ = 405.346


    Case 99     'NAD27 - CONUS
      .a = 6378206.4
      .b = .a * (1# - 1# / 294.9786982)

      .dx = -8#                    'Datumsuebergang NAD27 -> WGS84
      .dy = 160#
      .dZ = 176#


    Case Else
      GetDatum = True

    End Select

    'Rotation
    .rX = 0#
    .rY = 0#
    .rZ = 0#

    '----- Abgeleitete Ellipsenwerte -----
    'Quadrate der Halbachsen
    asq# = .a * .a
    bsq# = .b * .b

    'Abplattung  f = (a-b)/a = 1-b/a
    .f = (.a - .b) / .a

    'Polkruemmungshalbmesser   c = a^2/b = a/(1-f)
    .c = asq# / .b

    'Quadrat der 1. numerischen Exzentrizitaet
    '  e1sq = (a^2-b^2)/a^2 = 2*f-f^2 = 2*f-f*f = f*(2-f)
    .e1sq = (asq# - bsq#) / asq#

    '1. numerische Exzentrizitaet
    .e1 = Sqr(.e1sq)

    'Quadrat der 2. numerischen Exzentrizitaet
    '  e2sq = (a^2-b^2)/b^2 = e1sq/(1-e1sq)
    .e2sq = (asq# - bsq#) / bsq#

    '2. numerische Exzentrizitaet
    .E2 = Sqr(.e2sq)

  End With

End Function

Sub GetProjection(Mode, Proj As MapProjection)

  With Proj

    Select Case Mode
    Case 1                'UTM (Hayford)
      .e0 = 500000#
      .b0 = 0#
      .m = 0.9996

    Case 2                'Gauss-Krueger (Bessel)
      .e0 = 500000#
      .b0 = 0#
      .m = 1#

    Case 3                'Schweizer Gitter
      'dummy entry to avoid error

    Case Else
      Stop
    End Select

  End With

End Sub

Function Konf2Geo(ByVal MapDatum, ByVal MapCoSys, ByVal Rechts#, ByVal Hoch#, Longitude#, Latitude#)

  'MapDatum: 0=WGS84, 1=ED50, 2=Potsdam, 3=CH1903
  'MapCoSys: 0=Grad,  1=UTM,  2=Gau-Krger, 3=Schweizer Gitter
  'Rechts, Hoch: Werte in m
  'Longitude, Latitude: Lnge und Breite in Grad (Gradma)

  Dim Dat As GeoDatum
  Dim Proj As MapProjection
  Dim x As Double
  Dim Tmp As dGeoPoint

  Konf2Geo = True

  If MapCoSys = 0 Then Stop

  void = GetDatum(MapDatum, Dat)

  If MapCoSys = 3 Then
    '----- Schweizer Gitter - Nherungsformel -----
    'Quelle: Bundesamt fr Landestopographie - Eidgenssische Vermessungsdirektion
    '  "Formeln und Konstanten fr die Berechnung der Schweizerischen schiefachsigen
    '  Zylinderprojektion und der Transformation zwischen Koordinatensystemen - Januar 2001"
    '  ftp.swisstopo.ch/pub/data/geo/refsysd.pdf
    '  siehe Kapitel 3.4

    If Rechts# < 0 Or Hoch# < 0 Then Exit Function

    y# = Rechts# / 1000000# - 0.6    'Landeskoordinaten in 1000 km
    YSq# = y# * y#
    x = Hoch# / 1000000# - 0.2

    a1# = 4.72973056 + x * (0.7925714 + x * (0.132812 + x * 0.025 + x * x * 0.0048))
    a3# = -0.04427 + x * (-0.0255 - x * 0.0096)
    a5# = 0.00096
    Lambda# = 2.67825 + y# * (a1# + YSq# * (a3# + YSq# * a5#))   'in 10000"

    p0# = x * (3.23864877 + x * (-0.0025486 + x * (-0.013245 + x * 0.000048)))
    P2# = -0.27135379 + x * (-0.0450442 + x * (-0.007553 - x * 0.00146))
    p4# = 0.002442 + x * 0.00132
    Phi# = 16.902866 + p0# + YSq# * (P2# + YSq# * p4#)           'in 10000"

    Longitude# = Lambda# * 10000# / 3600#     'in 
    Latitude# = Phi# * 10000# / 3600#

  Else

    Call GetProjection(MapCoSys, Proj)

    n# = Hoch#                        'Northing
    Zone = Int(Rechts# / 1000000#)
    e# = Rechts# - Zone * 1000000#    'Easting

    Select Case MapCoSys
    Case 1                'UTM
      If Rechts# < 0 Then Exit Function
      'lR = Len(Format$(Rechts#))
      'If lR <> 7 And lR <> 8 Then Exit Function
      L0# = ((Zone - 1) * 6 - 180 + 3) * deg2rad    'Bezugsmeridian

    Case 2                'Gau-Krger DL
      If Rechts# < 0 Or Hoch# < 0 Then Exit Function
      'If Len(Format$(Rechts#)) <> 7 Then Exit Function
      L0# = Zone * 3 * deg2rad                      'Bezugsmeridian
    End Select

    On Error Resume Next
    Call en2bl(n#, e#, l#, b#, Dat.c, Dat.E2, Pi, L0#, Proj.e0, Proj.b0, Proj.m)
    ErrNr = Err
    On Error GoTo 0
    If ErrNr Then Exit Function

    Longitude# = l# * rad2deg          'geographische Laenge L
    Latitude# = b# * rad2deg          'geographische Breite B

  End If

  Konf2Geo = False

End Function

Function PosStr2dCoord(InMapDatum, MapCoSys, mb$, ml$, pDegOut As dGeoPoint)

  Dim pDeg As dGeoPoint, Konf As KonfCoord

  PosStr2dCoord = True

  If MapCoSys = 0 Then

    'Geogr.Koord. (Grad)
    ErrNr = Str2dDegCoord(mb$, pDeg.b, 1)
    If ErrNr Then Exit Function
    ErrNr = Str2dDegCoord(ml$, pDeg.l, 2)
    If ErrNr Then Exit Function

  Else

    'Konforme Koordinaten
    If MapCoSys = 1 Then
      'UTM
      NOSW$ = Msg$(40200)

      Select Case Left$(mb$, 1)
      Case Mid$(NOSW$, 1, 1)
        mb$ = Trim$(Mid$(mb$, 2))     'Remove "N"
      Case Mid$(NOSW$, 3, 1)
        mb$ = Trim$(Mid$(mb$, 2))     'Remove "S"
        SouthUtm = True
      End Select

      ml$ = Replace$(ml$, " ", "")    'Remove " "

    Else
      'Gau-Krger, Schweizer Gitter
      HR$ = Msg$(40201)
      If Left$(mb$, 1) = Mid$(HR$, 1, 1) Then mb$ = Trim$(Mid$(mb$, 2))     'Remove "H"
      If Left$(ml$, 1) = Mid$(HR$, 2, 1) Then ml$ = Trim$(Mid$(ml$, 2))     'Remove "R"
    End If

    ErrNr = Str2Dbl(ml$, Rechts#)
    If ErrNr Then Exit Function

    ErrNr = Str2Dbl(mb$, Hoch#)
    If ErrNr Then Exit Function
    'UTM false Northing for southern hemisphere
    If SouthUtm Then Hoch# = Hoch# - 10000000#

    ErrNr = Konf2Geo(InMapDatum, MapCoSys, Rechts#, Hoch#, pDeg.l, pDeg.b)
    If ErrNr Then Exit Function
  End If

  'Output always in WGS84
  PosStr2dCoord = CoordTransf(InMapDatum, pDeg, 0, pDegOut)

End Function

