Attribute VB_Name = "dbFile"
DefInt A-Z

Sub CreateDBFunctions(Index)

  Dim t$, Filt$, FileSpec$, Cancel, ErrNr, Found

  'Datenbank anlegen
  t$ = ProgName$ & " - " & StdMsg$(32023)
  Filt$ = "Access-Datenbank (*.MDB)|*.MDB"
  'Flags: PathMustExist + NoChangeDir + HideReadOnly + Overwrite-Prompt
  Cancel = GetFileSpec(t$, 2, &H80E, ".", "MDB", Filt$, FileSpec$)
  If Cancel Then Exit Sub

  ErrNr = CloseDB()
  If ErrNr Then Exit Sub

  ErrNr = DeleteFile(FileSpec$, Found)
  If ErrNr Then Exit Sub

  ErrNr = CreateDB(FileSpec$, Index)
  If ErrNr Then Exit Sub

  Call ShowSelection

End Sub

Sub ExportFunctions(FktNr, MsgNr)

  'Menu  SubMenu       FktNr   MsgNr   Format
  ' 0                    0       0     Access
  ' 1     0...2        1...3     1     dBase
  ' 2     0...4        4...8     2     EXCEL
  ' 3       0            9       3     CSV
  '       1...2        10,11     4     Text
  ' 4                                  ----
  ' 5...               12...           Stationen, Links, Stdte, ATV ...


  Dim t$, Ext$, Filt$, Flags&
  Dim Cancel, ErrNr, Found, Exist, MultiTableDB
  Dim DstFile$, DstDB$, DstTbl$, DstType$
  Dim ExtDB As Database

  If Not dbDynasetIsOpen Then Exit Sub
  If ds.RecordCount = 0 Then Exit Sub

  Select Case FktNr
  Case 0        'Access
    DstType$ = "Access"
    MultiTableDB = True

  Case 1 To 3   'dBASE
    DstType$ = StdMsg$(32080 + FktNr - 1)

  Case 4 To 5   'EXCEL 3.0, 4.0
    DstType$ = StdMsg$(32090 + FktNr - 4)

  Case 6 To 8   'EXCEL 5.0, 7.0, 8.0
    DstType$ = StdMsg$(32090 + FktNr - 4)
    MultiTableDB = True

  Case 9        'Text-Tabelle
    'Export-Format s. SCHEMA.INI file (VB5-Manual page 197 ff)
    DstType$ = "Text"

  Case Else    'Text-Datei...
    DstType$ = ""
  End Select

  t$ = ProgName$ & " - " & Msg$(32258)
  Ext$ = Msg$(32540 + MsgNr)
  Filt$ = Msg$(32530 + MsgNr) & " (*." & Ext$ & ")|*." & Ext$

  If MultiTableDB Then
    'Flags: PathMustExist + NoChangeDir + HideReadOnly
    Flags& = &H80C
  Else
    'Flags: PathMustExist + NoChangeDir + HideReadOnly + Overwrite-Prompt
    Flags& = &H80E
  End If

  Cancel = GetFileSpec(t$, 2, Flags&, ".", Ext$, Filt$, DstFile$)
  If Cancel Then Exit Sub

  ErrNr = GetConnectPar(DstFile$, DstType$, DstDB$, DstTbl$)
  If ErrNr Then Exit Sub

  If MultiTableDB Then
    ErrNr = GetExtTable(False, DstType$, DstFile$, DstTbl$)
  Else
    ErrNr = DeleteFile(DstFile$, Found)
  End If
  If ErrNr Then Exit Sub


  Call ShowMsg(Msg$(32258) & Msg$(32259), False, 11)

  Select Case FktNr
  Case 0 To 8   'Access, dBASE, EXCEL
    ErrNr = ExportDB(DstDB$, DstTbl$, DstType$)

  Case 9  'CSV-Datei
    ErrNr = ExportDB(DstDB$, DstTbl$, DstType$)

  Case 10  'Text-Datei (Komma getrennt)
    ErrNr = WriteDS(DstFile$, "", ",", "", 0)

  Case 11  'Text-Datei (TAB getrennt)
    ErrNr = WriteDS(DstFile$, "", Chr$(9), "", 0)

  End Select

  If ErrNr Then
    Call ShowMsg(Msg$(32280), False, 0)
  Else
    Call ShowMsg("Export erfolgreich beendet", False, 0)
  End If

End Sub

Sub FileFunctions(Index)

  Dim void, ErrNr, Found
  Dim Cancel, Title$, Filt$, FileSpec$

  Title$ = ProgName$ & " - " & StdMsg$(32020 + Index)

  Select Case Index
  Case 0  'Datenbank ffnen
    Filt$ = "Access-Datenbank (*.MDB)|*.MDB"
    'Flags: FileMustExist + PathMustExist + NoChangeDir + HideReadOnly
    Cancel = GetFileSpec(Title$, 1, &H180C, ".", "MDB", Filt$, FileSpec$)
    If Cancel Then Exit Sub

    If FileSpec$ <> dbOpenDatabase Then
      ErrNr = CloseDB()
      If ErrNr = 0 Then ErrNr = OpenDB(FileSpec$)
      If ErrNr = 0 Then ErrNr = SelectTable(True)
    End If


  Case 1  'Datenbank schlieen
    ErrNr = CloseDB()
    Call ShowSelection

  Case 2  '-

  Case 3  'Datenbank anlegen

  Case 4  'Datenbank komprimieren
    Call PackDatabase(Title$, Index)


  Case 5  'Datenbank reparieren
    Filt$ = "Access-Datenbank (*.MDB)|*.MDB"
    'Flags: FileMustExist + PathMustExist + NoChangeDir + HideReadOnly
    Cancel = GetFileSpec(Title$, 1, &H180C, ".", "MDB", Filt$, FileSpec$)
    If Cancel Then Exit Sub

    Call ShowMsg(StdMsg$(32020 + Index) & Msg$(32259), 0, 11)

    ErrNr = DAOCommand("RepairDB", FileSpec$, "", 32292)

    Call ShowSelection


  Case 6  '-

  Case 7  'Drucken
    Call PrintCommand

  Case 8  'Schriftart
    Call PrtSelectFont(FileSpecForm.CmDlg)

  Case 9  'Seite einrichten
    void = PrtPageSetup()

  Case 10  'Drucker einrichten
    Call PrinterSetup(FileSpecForm.CmDlg)

  Case 11  '-

  Case 12  'Log-Datei aufzeichnen
    If LogFileNr Then
      'LogFile schlieen
      LogFileSpec = ""
      ErrNr = CloseFile(LogFileNr)
    Else
      'Log-File ffnen
      Filt$ = "Log-Datei (*.LOG)|*.LOG"
      'Flags: PathMustExist + NoChangeDir + HideReadOnly + Overwrite-Prompt
      Cancel = GetFileSpec(Title$, 2, &H80E, ".", "LOG", Filt$, FileSpec$)

      If Not Cancel Then Call OpenLogFile(FileSpec$)
    End If

  Case 13  'SQL-Skript ausfhren
    Filt$ = "SQL-Skript (*.SQL)|*.SQL"
    'Flags: FileMustExist + PathMustExist + NoChangeDir + HideReadOnly
    Cancel = GetFileSpec(Title$, 1, &H180C, ".", "SQL", Filt$, FileSpec$)
    If Cancel Then Exit Sub

    MsgBox FileSpec$


  Case 14  'frei

  Case 15  '-

  Case 16  'Ende
    Call appQuit

  End Select

End Sub

Sub ImportFunctions(FktNr, MsgNr)

  'Menu  SubMenu       FktNr   MsgNr   Format
  ' 0                     0      0     Access
  ' 1     0...2         1...3    1     dBase
  ' 2     0...4         4...8    2     EXCEL
  ' 3                     9      3     CSV
  ' 4                                  ----
  ' 5                    10      4     HAMMAP-Stationen
  ' 6                    11      5     HAMMAP-Links
  ' 7                    12      6     Locator-Datenbank
  ' 8        0           13      7     DXCC Country.tab
  '          1           14      7     DXCC CountryDel.tab
  '          2           15      8     DXCC OK1RR.tbl

  Dim SrcType$, SrcFile$, SrcDB$, SrcTbl$
  Dim t$, Ext$, Filt$, Flags&, d$, p$, n$, e$
  Dim ImportMode, DstTbl$, Cancel, ErrNr
  Static DefDir$

  MultiTableDB = False

  Select Case FktNr
  Case 0       'Access
    SrcType$ = "Access"
    MultiTableDB = True
  Case 1 To 3  'dBASE
    SrcType$ = StdMsg$(32080 + FktNr - 1)
  Case 4 To 5  'EXCEL 3.0, 4.0
    SrcType$ = StdMsg$(32090 + FktNr - 4)
  Case 6 To 8  'EXCEL 5.0, 7.0, 8.0
    SrcType$ = StdMsg$(32090 + FktNr - 4)
    MultiTableDB = True
  Case 9  'Text-Tebelle
    'Import-Format s. SCHEMA.INI file (VB5-Manual page 197 ff)
    SrcType$ = "Text"
  Case 12
    SrcFile$ = "Locator.dbf"
  Case 13      'Country.tab
    SrcFile$ = "Country.tab"
    SrcType$ = ""
  Case 14      'CountryDel.tab
    SrcFile$ = "CountryDel.tab"
    SrcType$ = ""
  Case 15      'AreaOK1RR.tbl
    SrcFile$ = "AreaOK1RR.tbl"
    SrcType$ = ""
  Case Else    'HAMMAP-Stationen etc.
    SrcType$ = ""
  End Select

  t$ = ProgName$ & " - " & Msg$(32257)
  Ext$ = Msg$(32520 + MsgNr)
  Filt$ = Msg$(32510 + MsgNr) & " (*." & Ext$ & ")|*." & Ext$
  'Flags: FileMustExist + PathMustExist + NoChangeDir + HideReadOnly
  Flags& = &H180C

  Cancel = GetFileSpec(t$, 1, Flags&, DefDir$, Ext$, Filt$, SrcFile$)
  If Cancel Then Exit Sub

  If FktNr <> 12 Then

    Call ParseFileSpec(SrcFile$, d$, p$, n$, e$)
    DefDir$ = d$ & p$

    ErrNr = GetConnectPar(SrcFile$, SrcType$, SrcDB$, SrcTbl$)
    If ErrNr Then Exit Sub

    If MultiTableDB Then
      ErrNr = GetExtTable(True, SrcType$, SrcFile$, SrcTbl$)
      If ErrNr Then Exit Sub
    End If

    Cancel = GetInsertOptions(DstTbl$, ImportMode)
    If Cancel Then Exit Sub

    If ImportMode = 2 Then
      ErrNr = DeleteTable(DstTbl$, 32272)
      If ErrNr Then Exit Sub
    End If

  End If

  Call ShowMsg(Msg$(32257) & Msg$(32259), False, 11)

  Select Case FktNr
  Case 0 To 9
    ErrNr = ImportDB(SrcDB$, SrcTbl$, SrcType$, ImportMode, DstTbl$)

  Case 10  'Stationen
    ErrNr = ImportHMTable(SrcFile$, 32320, ",", ImportMode, DstTbl$)

  Case 11  'Links
    ErrNr = ImportHMTable(SrcFile$, 32340, ",", ImportMode, DstTbl$)

  Case 12  'Locator-Datenbank
    ErrNr = ImportLocator(SrcFile$)
    DstTbl$ = "tblLocator"

  Case 13  'Country.tab
    ErrNr = ImportHMTable(SrcFile$, 32350, "|", ImportMode, DstTbl$)

  Case 14  'CountryDel.tab
    ErrNr = ImportHMTable(SrcFile$, 32360, "|", ImportMode, DstTbl$)

  Case 15  'AreaOK1RR.tbl
    ErrNr = ImportHMTable(SrcFile$, 32380, "|", ImportMode, DstTbl$)

  End Select

  If ErrNr Then
    Call ShowMsg(Msg$(32281), False, 0)
  Else
    ErrNr = OpenTbl(DstTbl$)
    If ErrNr Then Exit Sub
    ErrNr = SelectData(SelectClause & FromClause)
    Call ShowSelection
  End If

End Sub

