Attribute VB_Name = "GETSTN"
'---------------------------- GETSTN.BAS ----------------------------

'Stations-Lade-Unterprogramme
'von Gerhard Berg, DJ6FM


DefInt A-Z


Function CmpCalls(Call1 As StationCall, Call2 As StationCall)

  If Call1.Call > Call2.Call Then
    CmpCalls = 1
  ElseIf Call1.Call < Call2.Call Then
    CmpCalls = 2
  Else
    If Call1.SSID > Call2.SSID Then
      CmpCalls = 1
    ElseIf Call1.SSID < Call2.SSID Then
      CmpCalls = 2
    Else
      CmpCalls = 0
    End If
  End If

End Function

Sub DecodeCall(ActCall As StationCall, OutCall$, LineNr$)

  OutCall$ = UnPackCall$(ActCall.Call)

  SSID$ = LTrim$(Str$(ActCall.SSID \ 256))
  If SSID$ > "0" Then OutCall$ = OutCall$ + "-" + SSID$

  LineNr$ = LTrim$(Str$(ActCall.SSID And 255))

End Sub

Function DecodeDate$(iDate, Mode)

  If iDate = 0 Then

    sDate$ = ""

  Else

    sDate$ = Format$(iDate, "0000")

    If Mode And 2 Then
      yy$ = Left$(sDate$, 2)
      If Val(yy$) > 80 Then
        sDate$ = Mid$(sDate$, 3) + ".19" + yy$
      Else
        sDate$ = Mid$(sDate$, 3) + ".20" + yy$
      End If
    End If

  End If

  DecodeDate$ = sDate$

End Function

Function DecodeLink(Lnk As LinkData, Mode)

  'Mode: Bit 0: 0 = unbekannt = ""
  '             1 = unbekannt = '?' (nur fuer Bit 1 = 0)
  '      Bit 1: 0 = Abkuerzung (2 Byte)
  '             2 = voller Text (Status, Typ)

  Static ActCall As StationCall

  DecodeLink = True

  '--- Calls decodieren ---
  For i = 1 To 2
    ErrNr = GetStationCall(Lnk.StnPtr(i), ActCall)
    If ErrNr Then Exit Function
    Call DecodeCall(ActCall, LF$(i), dummy$)
  Next i


  'Link-Datum
  LF$(4) = DecodeDate(Lnk.LnkDate, Mode)

  'Link-Typ
  LinkTyp$ = GetStnAbv$(6, Lnk.StatTyp \ 256)


  If (Mode And 2) = 0 Then              'Abkuerzung (2 Buchstaben)
    'Link-Status
    LF$(3) = GetStnAbv$(5, Lnk.StatTyp And 255)

    'Link-Typ = WI  ->  Link-Status = WI
    If (SV.DataFormat And 16) <> 0 And LinkTyp$ = "WI" Then
      LF$(3) = "WI"
    End If

    'Link-Typ
    LF$(6) = LinkTyp$

  Else                                  'Langer Text

    'Link-Status
    LF$(3) = Msg$(24280 + (Lnk.StatTyp And 255))

    'Link-Typ
    LF$(6) = Msg$(24260 + (Lnk.StatTyp \ 256))

  End If

  If (Mode And 1) = 0 Then
    If LF$(3) = "?" Then LF$(3) = ""
    If LF$(6) = "?" Then LF$(6) = ""
  End If


  '--- Link-Baud ---
  LF$(5) = Format$(Lnk.Baud, "#")

  'Link-Typ = WI  ->  Baud = 999999
  If (SV.DataFormat And 8) <> 0 And LinkTyp$ = "WI" Then
    LF$(5) = "999999"
  End If

  'Baud > 999999  ->  Baud = 999998 (max. 6 Stellen)
  If (SV.DataFormat And 64) <> 0 And Val(LF$(5)) > 999999 Then
    LF$(5) = "999998"
  End If

  DecodeLink = False

End Function

Sub DecodeStation(Stn As StationData, Qth$, Info$, Mode)

  'Mode: Bit 0: 0 = unbekannt = "", 1 = unbekannt = '?' (nur fuer Bit 1 = 0)
  '      Bit 1: 0 = Abkuerzung (2 Byte), 2 = voller Text (Status, Typ, SW, HW)


  'Format:
  'call,LineNr,[status],[datum],[typ],[software],[hardware],
  '  [qrg],[baud],[sysop],[locator],[qth],[info]

  Dim dCoord As dGeoPoint

  'Call
  Call DecodeCall(Stn.StnCall, SF$(1), SF$(2))

  'Datum
  SF$(4) = DecodeDate(Stn.StnDate, Mode)

  If (Mode And 2) = 0 Then              'Abkuerzung (2 Buchstaben)
    'Status
    SF$(3) = GetStnAbv$(1, Stn.StnStat)

    'Typ
    SF$(5) = GetStnAbv$(2, Stn.StnTyp)

    'Software
    SF$(6) = GetStnAbv$(3, Stn.Software)

    'Hardware
    SF$(7) = GetStnAbv$(4, Stn.Hardware)

  Else                       'Langer Text
    'Status
    SF$(3) = Msg$(24300 + Stn.StnStat)

    'Typ
    SF$(5) = Msg$(24310 + Stn.StnTyp)

    'Software
    SF$(6) = Msg$(24340 + Stn.Software)

    'Hardware
    SF$(7) = Msg$(24370 + Stn.Hardware)
  End If


  If (Mode And 1) = 0 Then
    If SF$(3) = "?" Then SF$(3) = ""
    If SF$(5) = "?" Then SF$(5) = ""
    If SF$(6) = "?" Then SF$(6) = ""
    If SF$(7) = "?" Then SF$(7) = ""
  End If


  'QRG  (returns '.' as decimal separator - no 1000 separators)
  SF$(8) = Format$(CCur(Stn.QRG) / 1000@, "0.000;-0.000;#")
  l = Len(SF$(8))
  If l <> 0 Then Mid$(SF$(8), l - 3, 1) = "."

  'Baud
  SF$(9) = Format$(Stn.Baud, "#")

  'Sysop
  SF$(10) = UnPackCall$(Stn.Sysop)

  'Locator
  'If xgeo >= 32766 Or ygeo = 32767 Then   '*** Fehler in V8.1 ***
  If Stn.xloc >= 32766 Or Stn.yloc = 32767 Then
    'Station gelscht(x=32767), Zusatzzeile(x=32766) oder QTH unbekannt(y=32767)
    SF$(11) = ""
  ElseIf (Stn.StnCall.SSID And 255) <> 0 Then
    'Zusatzzeile (falls x nicht gesetzt wg. bug)
    SF$(11) = ""
  Else
    dCoord.b = Stn.yloc * cfQth2Deg
    dCoord.l = Stn.xloc * cfQth2Deg
    void = dDegCoord2Qth(dCoord, SF$(11))
    SF$(11) = Left$(SF$(11), 6)
  End If

  'QTH
  SF$(12) = Qth$

  'Info
  SF$(13) = Info$


  '--- Ungueltige Felder in Zusatzzeilen loeschen (ausser Call und Zeile) ---
  If Val(SF$(2)) <> 0 Then
    For i = 3 To AnzStnFields
      If (2 ^ (i - 3) And SV.AddLines) = 0 Then SF$(i) = ""
    Next i
  End If

End Sub

Function EncodeCall(Mode, InCall$, LineNr$, StnCall As StationCall)

  EncodeCall = True

  '--- Rufzeichen ---
  SSID$ = InCall$
  sCall$ = Parse$(SSID$, "-")

  ErrNr = PackCall(Mode, sCall$, StnCall.Call)
  If ErrNr Then Exit Function


  '--- SSID ---
  If Len(SSID$) <= 2 Then
    SsidVal = Val(SSID$)
    StnCall.SSID = 256 * SsidVal
  Else
    SsidVal = -1
  End If

  If SsidVal < 0 Or SsidVal > 15 Then
    ErrNr = StnLnkError(Mode, 31031, InCall$)
    If ErrNr Then Exit Function
  End If


  '--- Zeile ---
  Select Case LineNr$
  Case ""
  Case "0" To "9"
    StnCall.SSID = StnCall.SSID Or Val(LineNr$)
  Case Else
    ErrNr = StnLnkError(Mode, 31002, LineNr$)
    If ErrNr Then Exit Function
  End Select

  EncodeCall = False

End Function

Function FindStation(FindCall As StationCall, StnNr, RecNr, Found, Deleted)

  Static Stn As StationData
  Static MidCall As StationCall

  FindStation = True
  Found = False

  AnzStnRec = SV.Out.StnHdr.AnzStnRec

  '--- Stations-Index nicht geladen -> sequentiell suchen ---
  If SV.StnIdxLoaded = 0 Then
    For RecNr = 2 To AnzStnRec
      ErrNr = GetStationData(SV.Out.BsdFile, RecNr, Stn)
      If ErrNr Then Exit Function
      If Stn.StnCall.Call = FindCall.Call Then
        If Stn.StnCall.SSID = FindCall.SSID Then
          StnNr = RecNr
          Found = True
          Deleted = (Stn.xloc = 32767)
          FindStation = False
          Exit Function
        End If
      End If
    Next RecNr
    RecNr = AnzStnRec + 1
    FindStation = False     'Station nicht gefunden
    Exit Function
  End If


  '--- Stations-Index ist geladen -> Binaersuche ---
  Lo = 1                                        'Skip Header = #1
  Hi = AnzStnRec + 1
  Diff = Hi - Lo

  '--- Station suchen ---
  Do Until Diff = 1
    Mitte = Diff \ 2 + Lo
    RecNr = StnIdxTbl(Mitte)

    ErrNr = GetStationCall(RecNr, MidCall)
    If ErrNr Then Exit Function

    Select Case CmpCalls(FindCall, MidCall)
    Case 1      'FindCall > MidCall
      Lo = Mitte

    Case 2      'FindCall < MidCall
      Hi = Mitte

    Case Else      'FindCall = MidCall

      '--- Station gefunden ---
      StnNr = Mitte
      RecNr = StnIdxTbl(Mitte)
      ErrNr = GetStationQth(RecNr, x, y)
      If ErrNr Then Exit Function
      Deleted = (x = 32767)
      Found = True
      FindStation = False
      Exit Function
    End Select

    Diff = Hi - Lo
  Loop

  '--- Station nicht gefunden ---
  StnNr = Hi
  RecNr = StnNr
  FindStation = False

End Function

Function FindStnAbv(AbvNr, Abv$)

  FindStnAbv = (InStr(StnAbv(AbvNr), UCase$(Abv$)) - 1) \ 3

End Function

Function GetLinkCalls(RecNr, StnCall1 As StationCall, StnCall2 As StationCall)

  GetLinkCalls = True

  ErrNr = GetLnkStnPtrs(RecNr, StnPtr1, StnPtr2)
  If ErrNr Then Exit Function

  ErrNr = GetStationCall(StnPtr1, StnCall1)
  If ErrNr Then Exit Function

  ErrNr = GetStationCall(StnPtr2, StnCall2)
  If ErrNr Then Exit Function

  GetLinkCalls = False

End Function

Function GetLinkData(FileNr, RecNr, Lnk As LinkData)

  Static LastFileNr
  Static LastRecNr
  Static LastLnk As LinkData


  If SV.LnkDataLoaded Then

    '---Link-Daten aus RAM einlesen ---
    If RecNr > 0 Then Lnk = LnkTbl(RecNr)

  Else

    If RecNr = 0 Then

      '--- Puffer loeschen ---
      LastFileNr = 0
      LastRecNr = 0

    ElseIf RecNr < 0 Then

      '--- Puffer aktualisieren ---
      LastFileNr = FileNr
      LastRecNr = -RecNr
      LastLnk = Lnk

    Else

      '---Link-Daten aus Datei einlesen ---
      If FileNr = LastFileNr And RecNr = LastRecNr Then
        Lnk = LastLnk
      Else
        ErrNr = InputLinkData(FileNr, RecNr, Lnk)
        If ErrNr Then Exit Function

        LastFileNr = FileNr
        LastRecNr = RecNr
        LastLnk = Lnk
      End If
    End If
  End If

  'Ungueltigen Status auf unbekannt setzen
  'If (Lnk.StatTyp And 255) >= AnzStnAbv(5) Then
  '  Lnk.StatTyp = Lnk.StatTyp And &H7F00
  'End If

  'Ungueltigen Typ auf 0 = unbekannt setzen
  'If (Lnk.StatTyp \ 256) >= AnzStnAbv(6) Then
  '  Lnk.StatTyp = Lnk.StatTyp And &H00FF
  'End If

  GetLinkData = False

End Function

Function GetLnkStnPtrs(RecNr, StnPtr1, StnPtr2)

  Static Lnk As LinkData

  GetLnkStnPtrs = False

  If SV.LnkStnPtrLoaded Then
    StnPtr1 = LnkStnPtrTbl(RecNr).StnPtr(1)
    StnPtr2 = LnkStnPtrTbl(RecNr).StnPtr(2)
  ElseIf SV.LnkDataLoaded Then
    StnPtr1 = LnkTbl(RecNr).StnPtr(1)
    StnPtr2 = LnkTbl(RecNr).StnPtr(2)
  Else
    GetLnkStnPtrs = InputLinkData(SV.Out.BldFile, RecNr, Lnk)
    StnPtr1 = Lnk.StnPtr(2)
    StnPtr2 = Lnk.StnPtr(2)
  End If

End Function

Function GetNextLink(LnkRec, NextLink, SrcRec, DstRec, Lnk As LinkData)

  '--- Linkdaten einlesen ---
  GetNextLink = GetLinkData(SV.Out.BldFile, LnkRec, Lnk)

  '--- Link-Partner und naechsten Link bestimmen ---
  Ptr = Lnk.StnPtr(1) = SrcRec
  DstRec = Lnk.StnPtr(1 - Ptr)
  NextLink = Lnk.LnkPtr(2 + Ptr)

End Function

Function GetStationCall(RecNr, StnCall As StationCall)

  Static Stn As StationData

  GetStationCall = False

  If SV.StnDataLoaded Then
    StnCall = StnTbl(RecNr).StnCall
  ElseIf SV.StnCallsLoaded Then
    StnCall = StnCallTbl(RecNr)
  Else
    GetStationCall = InputStationData(SV.Out.BsdFile, RecNr, Stn)
    StnCall = Stn.StnCall
  End If

End Function

Function GetStationData(FileNr, RecNr, Stn As StationData)

  Static LastFileNr
  Static LastRecNr
  Static LastStn As StationData


  If SV.StnDataLoaded Then

    '--- Stations-Daten aus RAM lesen ---
    If RecNr > 0 Then Stn = StnTbl(RecNr)

  Else

    If RecNr = 0 Then

      '--- Puffer loeschen ---
      LastFileNr = 0
      LastRecNr = 0

    ElseIf RecNr < 0 Then

      '--- Puffer aktualisieren ---
      LastFileNr = FileNr
      LastRecNr = -RecNr
      LastStn = Stn

    Else

      '--- Stations-Daten aus Datei lesen ---
      If FileNr = LastFileNr And RecNr = LastRecNr Then
        Stn = LastStn
      Else
        ErrNr = InputStationData(FileNr, RecNr, Stn)
        If ErrNr Then Exit Function
        LastFileNr = FileNr
        LastRecNr = RecNr
        LastStn = Stn
      End If
    End If
  End If

  'Ungueltigen Status, Typ, SW, HW auf 0 = unbekannt setzen
  'If Stn.StnStat >= AnzStnAbv(1) Then Stn.StnStat = 0
  'If Stn.StnTyp >= AnzStnAbv(2) Then Stn.StnTyp = 0
  'If Stn.Software >= AnzStnAbv(3) Then Stn.Software = 0
  'If Stn.Hardware >= AnzStnAbv(4) Then Stn.Hardware = 0

  GetStationData = False

End Function

Function GetStationInfo(FileNr, Stn As StationData, Qth$, Info$)

  GetStationInfo = True

  '--- QTH aus Datei lesen ---
  If Stn.QthPtr = 0 Then
    Qth$ = ""
  Else
    ErrNr = GetLenText(FileNr, Stn.QthPtr, Qth$)
    If ErrNr Then Exit Function
  End If

  '--- Info aus Datei lesen ---
  If Stn.InfoPtr = 0 Then
    Info$ = ""
  Else
    ErrNr = GetLenText(FileNr, Stn.InfoPtr, Info$)
    If ErrNr Then Exit Function
  End If

  GetStationInfo = False

End Function

Function GetStationLineData(FileNr, StnRecNr)

  'Daten aller Zeilen laden (RecNr = Hauptzeile)

  Static Stn As StationData
  Static LastRecNr, LastFileNr

  GetStationLineData = True

  If StnRecNr < 0 Then
    'Daten wurden geaendert, Puffer stimmt nicht mehr
    LastRecNr = 0
    LastFileNr = 0

  ElseIf LastRecNr = StnRecNr And LastFileNr = FileNr Then
    'Daten werden nicht geaendert

  Else
    LastRecNr = StnRecNr
    LastFileNr = FileNr

    'Alle Zeilen als geloescht markieren
    For i = 0 To 9
      StnLineData(i).xloc = 32767
    Next i

    'StnRecNr darf nicht veraendert werden
    RecNr = StnRecNr

    Do Until RecNr = 0
      ErrNr = GetStationData(FileNr, RecNr, Stn)
      If ErrNr Then Exit Function

      LineNr = Stn.StnCall.SSID And 255
      StnLineData(LineNr) = Stn

      RecNr = Stn.AddLnPtr
    Loop
  End If

  GetStationLineData = False

End Function

Function GetStationQth(RecNr, x, y)

  Dim Stn As StationData

  GetStationQth = False

  If SV.StnDataLoaded Then
    x = StnTbl(RecNr).xloc
    y = StnTbl(RecNr).yloc
  ElseIf SV.StnQthLoaded Then
    x = StnQthTbl(RecNr).x
    y = StnQthTbl(RecNr).y
  Else
    GetStationQth = InputStationData(SV.Out.BsdFile, RecNr, Stn)
    x = Stn.xloc
    y = Stn.yloc
  End If

End Function

Function GetStnAbv$(AbvNr, Offset)

  GetStnAbv$ = UCase$(RTrim$(Mid$(StnAbv(AbvNr), 3 * Offset + 1, 2)))

End Function

Function GetStnIdx(StnNr)

  If SV.StnIdxLoaded Then

    GetStnIdx = StnIdxTbl(StnNr)

  Else

    GetStnIdx = StnNr

  End If

End Function

Function GetStnVer()

  GetStnVer = 0

  ErrNr = LoadTextFile(HomeDir$ + "STN\STNVER.BAT", 100, FileData$)
  If ErrNr Then Exit Function

  Ver$ = Parse$(FileData$, Chr$(13))

  If InStr(Ver$, "=") Then dummy$ = Parse$(Ver$, "=")

  GetStnVer = Val(Ver$)

End Function

Function InputLinkData(FileNr, RecNr, Lnk As LinkData)

  On Local Error Resume Next

  Get #FileNr, RecNr, Lnk

  InputLinkData = Err

  If Err Then Call ShowErrNrFileNr(Err, FileNr)

End Function

Function InputLinkHeader(FileNr, LnkHdr As LinkHeader)

  On Local Error Resume Next

  Get #FileNr, 1, LnkHdr

  InputLinkHeader = Err

  If Err Then Call ShowErrNrFileNr(Err, FileNr)

End Function

Function InputStationData(FileNr, RecNr, Stn As StationData)

  On Local Error Resume Next

  Get #FileNr, RecNr, Stn

  InputStationData = Err

  If Err Then Call ShowErrNrFileNr(Err, FileNr)

End Function

Function InputStationHeader(FileNr, StnHdr As StationHeader)

  On Local Error Resume Next

  Get #FileNr, 1, StnHdr

  InputStationHeader = Err

  If Err Then Call ShowErrNrFileNr(Err, FileNr)

End Function

Function PackCall(Mode, InCall$, OutCall As Long)

  'Mode: 1=Station-Call, -1=LinkCall

  Static sCall As String * 6
  Dim pCall As Long

  PackCall = True

  CallLen = Len(InCall$)

  If CallLen > 0 And (CallLen < 2 Or CallLen > 6) Then
    BadCall = True
  Else
    sCall = UCase$(InCall$)

    pCall = 0&

    For i = 1 To 6

      ch$ = Mid$(sCall$, i, 1)

      Select Case ch$
      Case " "
        n = 0
      Case "0" To "9"
        'ASC("0") = 48 -> 1...10
        n = Asc(ch$) - 47
      Case "A" To "Z"
        'ASC("A") = 65 -> 11...36
        n = Asc(ch$) - 54
      Case "?"
        n = 37
      Case "."
        n = 38
      Case Else
        BadCall = True
        Exit For
      End Select

      If i = 1 Then n = n - 20
      pCall = pCall * 40& + n

    Next i

    OutCall = pCall

  End If

  If BadCall Then
    ErrNr = StnLnkError(Mode, 31001, sCall$)
    If ErrNr Then Exit Function
  End If

  PackCall = False

End Function

Sub StoreLinkData(RecNr, Lnk As LinkData)

  '--- Link-Daten im RAM speichern ---
  If SV.LnkDataLoaded Then LnkTbl(RecNr) = Lnk

  If SV.LnkStnPtrLoaded Then
    LnkStnPtrTbl(RecNr).StnPtr(1) = Lnk.StnPtr(1)
    LnkStnPtrTbl(RecNr).StnPtr(2) = Lnk.StnPtr(2)
  End If

End Sub

Sub StoreStationData(RecNr, Stn As StationData)

  '--- Stations-Daten im RAM speichern ---
  If SV.StnDataLoaded Then StnTbl(RecNr) = Stn
  If SV.StnCallsLoaded Then StnCallTbl(RecNr) = Stn.StnCall

  If SV.StnQthLoaded Then
    StnQthTbl(RecNr).x = Stn.xloc
    StnQthTbl(RecNr).y = Stn.yloc
  End If

End Sub

Function UnPackCall$(ByVal pCall As Long)

  s$ = ""

  If pCall < 0 Then
    '20 * 40^5
    pCall = pCall + 2048000000
    Flag = True
  End If


  For i = 1 To 6

    ch = pCall Mod 40&
    pCall = pCall \ 40&

    If i = 6 And Flag = 0 Then ch = ch + 20

    Select Case ch
    Case 0
      's$ = " " + s$
    Case 1 To 10
      '1 -> ASC("0") = 48
      s$ = Chr$(ch + 47) + s$
    Case 11 To 36
      '11 -> ASC("A") = 65
      s$ = Chr$(ch + 54) + s$
    Case 37
      s$ = "?" + s$
    Case 38
      s$ = "." + s$
    Case Else
      s$ = "*" + s$
    End Select

  Next i

  UnPackCall$ = Trim$(s$)

End Function

