Attribute VB_Name = "PRTLIB_W"
'Universelle Druck-Routinen

DefInt A-Z
Option Explicit

'Orientation
Public Const Portrait = 1
Public Const Landscape = 2

'Printer Margins
Const PM_Top = 1
Const PM_Bottom = 2
Const PM_Left = 3
Const PM_Right = 4

Const AnzOpt = 4            'Anzahl Options

Type PrtType
  Installed As Long         'Drucker ist installiert
  SheetWidth As Single
  SheetHeight As Single
  PixelPerCmX As Single
  PixelPerCmY As Single

  Orientation As Long       '1=Portrait, 2=Landscape
  Margin(1 To 4) As Single  '1=Top, 2=Bottom, 3=Left, 4=Right
  PrintWidth As Single      'Druckbreite

  FontName As String
  FontSize As Single
  FontBold As Integer
  FontItalic As Integer

  ListTitle As String       'Titel fr DruckListe
  OptValue(1 To AnzOpt) As Long
  OptEnbl(1 To AnzOpt) As Long
  OptCaption(1 To AnzOpt) As String

  PageCount As Long         'Seiten-Nummer
End Type

Public Prt As PrtType

  Declare Function DrawText Lib "user32" Alias "DrawTextA" (ByVal hDC As Long, ByVal lpStr As String, ByVal nCount As Long, lpRect As RECT, ByVal wFormat As Long) As Long

Sub AbortPrint()

  'Druck abbrechen


  Printer.KillDoc


End Sub

Function OpenPrinter()

  Dim ErrNr

  On Error Resume Next
  Printer.Print ""              'Start Printer
  ErrNr = Err
  On Error GoTo 0

  If ErrNr Then Call ShowErrNr(ErrNr)

  OpenPrinter = ErrNr

End Function

Sub PrinterSetup(Cmdlg As Control)

  '--- Druckereinrichtung aufrufen ---

  Dim Cancel

  'Release hDC for printer (see Language Reference p. 455)
  Printer.EndDoc

  With Cmdlg
    .CancelError = True
    .PrinterDefault = True
    .Flags = &H40&

    On Error Resume Next
    .Action = 5
    Cancel = Err               'Abbruch
    On Error GoTo 0
  End With

  If Cancel = 0 Then
    'Force using new default printer
    Printer.TrackDefault = False
    Printer.TrackDefault = True
    Call PrtInit
  End If

End Sub

Function PrintField(ByVal m$, Align, Border, w!, d!) As Single

  'm$ = text to print
  'Align: 1=centered; 2 = right justified
  'Border: 1=Left; 2=Top; 4=Right; 8=Bottom
  'w! = Width in cm
  'd! = text distance from field border
  'Return value: Print Height in cm

  Dim l!, t!, r!, b!
  Dim dw!, dh!, th!
  Dim rc As RECT
  Dim Fmt, drawh&

  Fmt = (DT_SINGLELINE Or DT_NOPREFIX) Xor Align

  With Printer

    th! = .TextHeight(m$)             'Height
    dw! = 1 / Prt.PixelPerCmX         'Dot-Width
    dh! = 1 / Prt.PixelPerCmY         'Dot-Height

    If Len(m$) > 0 Then
      '--- Text drucken ---
      l! = .CurrentX - .ScaleLeft     'Left
      t! = .CurrentY - .ScaleTop      'Top
      If Prt.OptValue(4) Then t! = t! + dh!
      rc.Left = (l! + d!) * Prt.PixelPerCmX
      rc.Top = t! * Prt.PixelPerCmY
      rc.Right = (l! + w! - d!) * Prt.PixelPerCmX + 1
      rc.Bottom = (t! + th!) * Prt.PixelPerCmY + 1
      drawh& = DrawText(.hDC, m$, Len(m$), rc, Fmt)
    End If

    l! = .CurrentX      'Left
    r! = l! + w!        'Right
    t! = .CurrentY      'Top
    b! = t! + th!       'Bottom

    If Prt.OptValue(4) Then
      '--- Rahmen drucken ---
      b! = b! + 2 * dh!
      If Border And 1 Then Printer.Line (l!, t!)-(l!, b!)   'Left
      If Border And 2 Then Printer.Line (l!, t!)-(r!, t!)   'Top
      If Border And 4 Then Printer.Line (r!, t!)-(r!, b!)   'Right
      If Border And 8 Then Printer.Line (l!, b!)-(r!, b!)   'Bottom
    End If

    .CurrentX = r!
    .CurrentY = t!

    PrintField = b! - t!

  End With

End Function

Function PrintSection(s$)

  Dim r As RECT
  Dim Fmt, l!, h&

  If PrtEndOfPage() Then Call PrtStartPage("")

  If Right$(s$, 2) = vbCrLf Then
    s$ = Left$(s$, Len(s$) - 2)
  End If

  If Len(s$) = 0 Then
    Printer.Print
  Else
    With Printer
      .CurrentX = 0
      l! = -.ScaleLeft
      r.Left = l! * Prt.PixelPerCmX
      r.Top = (.CurrentY - .ScaleTop) * Prt.PixelPerCmY
      r.Right = (l! + Prt.PrintWidth) * Prt.PixelPerCmX
      r.Bottom = r.Top
      Fmt = DT_WORDBREAK Or DT_NOPREFIX

      'Test, ob Absatz noch auf aktuelle Seite pat
      h& = DrawText(.hDC, s$, Len(s$), r, Fmt Or DT_CALCRECT)
      If r.Bottom / Prt.PixelPerCmY > .ScaleHeight - Prt.Margin(PM_Bottom) Then
        Call PrtStartPage("")
        r.Top = (.CurrentY - .ScaleTop) * Prt.PixelPerCmY
        r.Bottom = r.Top + h&
      End If

      'Absatz Drucken
      h& = DrawText(.hDC, s$, Len(s$), r, Fmt)

      .CurrentY = .CurrentY + h& / Prt.PixelPerCmY
    End With

  End If

  PrintSection = False

End Function

Function PrtEndOfPage() As Integer

  'Prfen, ob Seitenende erreicht ist

  Dim lh As Single                        'Line Height
  Dim ph As Single                        'Print Height

  With Printer
    lh = .TextHeight("X")
    ph = .ScaleHeight - Prt.Margin(PM_Top) - Prt.Margin(PM_Bottom)

    PrtEndOfPage = .CurrentY + 2 * lh > ph
  End With

End Function

Sub PrtInit()

  Dim i, ErrNr

  '--- Prfen, ob Drucker installiert ---
  On Error Resume Next
  i = Printer.ScaleMode
  ErrNr = Err
  On Error GoTo 0

  Prt.Installed = (ErrNr = 0)

  If ErrNr Then
    MsgBox "Kein Standarddrucker installiert", 48
    Exit Sub
  End If


  '--- Drucker-Eigenschaften speichern ---
  With Printer
    Prt.PixelPerCmX = 567 / .TwipsPerPixelX
    Prt.PixelPerCmY = 567 / .TwipsPerPixelY

    .ScaleMode = 7      'cm
    Prt.SheetWidth = .ScaleWidth
    Prt.SheetHeight = .ScaleHeight
  End With


  '--- Drucker-Einstellungen initialisieren ---

  'Orientierung
  If Prt.Orientation = 0 Then
    Prt.Orientation = Portrait
  End If

  'Rnder
  If Prt.Orientation = Portrait Then
    'Hochformat
    Prt.Margin(PM_Top) = 2
    Prt.Margin(PM_Bottom) = 2
    Prt.Margin(PM_Left) = 2.5
    Prt.Margin(PM_Right) = 1.5
  Else
    'Querformat
    Prt.Margin(PM_Top) = 2.5
    Prt.Margin(PM_Bottom) = 1.5
    Prt.Margin(PM_Left) = 2
    Prt.Margin(PM_Right) = 2
  End If


  'Font
  If Len(Prt.FontName) = 0 Then Prt.FontName = "Arial"
  If Prt.FontSize = 0 Then Prt.FontSize = 8

End Sub

Function PrtPageSetup()

  'Seite einrichten

  Dim i, l, y, dy
  Dim ShowOpt
  Dim s As String


  '--- Anfangswerte setzen ---
  Load frmPageSetup

  With frmPageSetup

    y = .Txt(1).Top
    dy = 600

    'Controls laden
    For i = 2 To 4
      y = y + dy

      Load .Txt(i)
      .Txt(i).Top = y
      .Txt(i).Visible = True

      Load .Spn(i)
      .Spn(i).Top = y
      .Spn(i).Height = 375
      .Spn(i).Width = 255
      .Spn(i).Visible = True

      Load .Lbl(i)
      .Lbl(i).Top = y
      .Lbl(i).Visible = True
    Next i

    .Lbl(2).Caption = "Unten"
    .Lbl(3).Caption = "Links"
    .Lbl(4).Caption = "Rechts"

    'Seitenrnder bernehmen
    For i = 1 To 4
      .Txt(i).text = Format$(Prt.Margin(i), "0.0")
    Next i

    'Orientierung
    For i = 1 To 2
      .OrientOption(i).Value = (Prt.Orientation = i)
    Next i

    'Titel
    .TxtListTitle.text = Prt.ListTitle

    'Optionen
    For i = 1 To AnzOpt
      .Opt(i).Visible = Prt.OptEnbl(i)
      If Prt.OptEnbl(i) Then
        .Opt(i).Value = Prt.OptValue(i)
        If Len(Prt.OptCaption(i)) > 0 Then
          .Opt(i).Caption = Prt.OptCaption(i)
        End If
        ShowOpt = ShowOpt + 1
      End If
    Next i

    If ShowOpt < 4 Then
      ShowOpt = 0
      For i = 1 To AnzOpt
        If Prt.OptEnbl(i) Then
          .Opt(i).Top = 4200 + ShowOpt * 480
          ShowOpt = ShowOpt + 1
        End If
      Next i
    End If

    Call ShowMon

    '--- Eingabe ---
    .Show 1

    If .Tag = "0" Then

      '--- kein Abbruch -> neue Werte auslesen ---

      'Seitenrnder
      For i = 1 To 4
        s = Unformat$(.Txt(i).text)
        Prt.Margin(i) = CInt(Val(s) * 10) / 10
      Next i

      'Orientierung
      If .OrientOption(1).Value Then
        Prt.Orientation = Portrait
      Else
        Prt.Orientation = Landscape
      End If

      'Titel
      Prt.ListTitle = .TxtListTitle.text

      'Optionen
      For i = 1 To AnzOpt
        Prt.OptValue(i) = .Opt(i).Value
      Next i

    End If

  End With

  Unload frmPageSetup
  Set frmPageSetup = Nothing

End Function

Sub PrtSelectFont(Cmdlg As Control)

  'Drucker-Schriftart auswhlen

  Dim ErrNr As Integer

  With Cmdlg
    .CancelError = True
    .Flags = &H10402

    .FontName = Prt.FontName
    .FontSize = Prt.FontSize
    .FontBold = Prt.FontBold
    .FontItalic = Prt.FontItalic
  End With

  On Error Resume Next
  Cmdlg.Action = 4
  If Err Then Exit Sub
  On Error GoTo 0

  With Cmdlg
    On Error Resume Next
    Printer.FontName = .FontName
    Printer.FontSize = .FontSize
    Printer.FontBold = .FontBold
    Printer.FontItalic = .FontItalic
    ErrNr = Err
    On Error GoTo 0

    If ErrNr Then
      MsgBox "Unzulssige Schriftart."
    Else
      Prt.FontName = .FontName
      Prt.FontSize = .FontSize
      Prt.FontBold = .FontBold
      Prt.FontItalic = .FontItalic
    End If
  End With

End Sub

Function PrtSetOrientation(NewMode As Long)

  'NewMode: 1 = Portrait; 2 = Landscape

  Dim ErrNr


  On Error Resume Next
  Printer.Orientation = NewMode
  ErrNr = Err
  On Error GoTo 0

  If ErrNr Then Call ShowErrNr(ErrNr)


  PrtSetOrientation = ErrNr

End Function

Function PrtStartDoc()

  Dim ErrNr

  'Ausgabe eines Dokuments initialisieren
  PrtStartDoc = True

  'Orientierung
  On Error Resume Next
  Printer.EndDoc            'Release Printer
  On Error GoTo 0

  ErrNr = PrtSetOrientation(Prt.Orientation)
  If ErrNr Then Exit Function

  ErrNr = OpenPrinter()
  If ErrNr Then Exit Function

  With Printer
    'Drucker-Font einstellen
    .FontName = Prt.FontName
    .FontSize = Prt.FontSize
    .FontBold = Prt.FontBold

    'ich wei nicht warum, aber mit den zwei Befehlen
    'werden nderungen in der Schriftart bernommen
    'und sonst nicht
    .FontBold = True
    .FontBold = Prt.FontBold

    .FontItalic = Prt.FontItalic

    .ScaleMode = 7                     'cm
    .ScaleLeft = -Prt.Margin(PM_Left)
    .ScaleTop = -Prt.Margin(PM_Top)

    Prt.PrintWidth = .ScaleWidth - Prt.Margin(PM_Left) - Prt.Margin(PM_Right)

    .CurrentX = 0
    .CurrentY = 0
  End With

  'Seitenzhler lschen
  Prt.PageCount = 0

  PrtStartDoc = False

End Function

Sub PrtStartPage(OptTitle As String)

  'neue Seite Starten

  Dim NewLineFlag, SaveBold
  Dim m$, h!

  With Printer

    Prt.PageCount = Prt.PageCount + 1

    If Prt.PageCount > 1 Then .NewPage

    'Seiten-Nr.
    If Prt.OptValue(1) Then
      .CurrentX = 0
      m$ = "Seite" + Str$(Prt.PageCount)
      h! = PrintField(m$, 0, 0, Prt.PrintWidth, 0)
      NewLineFlag = True
    End If

    'Titel
    If Len(Prt.ListTitle) > 0 Then
      On Error Resume Next
      SaveBold = .FontBold
      .FontBold = True
      On Error GoTo 0
      .CurrentX = 0
      h! = PrintField(Prt.ListTitle, 1, 0, Prt.PrintWidth, 0)
      .FontBold = SaveBold
      NewLineFlag = True
    End If

    'Datum
    If Prt.OptValue(2) Then
      .CurrentX = 0
      h! = PrintField(Format$(Now, "dd/mm/yy"), 2, 0, Prt.PrintWidth, 0)
      NewLineFlag = True
    End If

    If NewLineFlag Then
      .CurrentX = 0
      .CurrentY = .CurrentY + 2 * h!
    End If

    'Suchkriterien und Sortierung
    If Prt.OptValue(3) And Len(OptTitle) > 0 Then
      Printer.Print OptTitle
      Printer.Print
    End If

  End With

End Sub

Sub ShowMon()

  'Monitorausgabe fr Druckseite mit Rndern

  Dim x1!, y1!, x2!, y2!

  With frmPageSetup
    .Pict.BackColor = &H80000005
  .Pict.FillColor = &H8000000F
  .Pict.FillStyle = 0      'solid

    If .OrientOption(1).Value Then
      'Portrait
      If Prt.SheetWidth > Prt.SheetHeight Then
        SWAP Prt.SheetWidth, Prt.SheetHeight
      End If
    Else
      'Landscape
      If Prt.SheetWidth < Prt.SheetHeight Then
        SWAP Prt.SheetWidth, Prt.SheetHeight
      End If
    End If

    .Pict.Width = Prt.SheetWidth * 50
    .Pict.Height = Prt.SheetHeight * 50

    .Pict.Scale (0, 0)-(Prt.SheetWidth, Prt.SheetHeight)

    .Pict.Cls

    x1! = Val(Unformat$(.Txt(3)))
    x2! = Prt.SheetWidth - Val(Unformat$(.Txt(4)))
    y1! = Val(Unformat$(.Txt(1)))
    y2! = Prt.SheetHeight - Val(Unformat$(.Txt(2)))

    .Pict.Line (x1!, y1!)-(x2!, y2!), , B
  End With

End Sub

