Attribute VB_Name = "QRGSUB"
'-------------------------- QRGSUB.BAS ------------------------

'HAMMAP QRG-Unterprogramme
'von Gerhard Berg, DJ6FM

DefInt A-Z


Dim QRGTbl() As QRGTable

Function GetChannel$(QRG&, i)

  'QRG& in 100 Hz
  Fmt$ = String$(QRGTbl(i).ChLen, "0")

  Channel = (QRG& - QRGTbl(i).QRGLow) / QRGTbl(i).Raster
  Channel = QRGTbl(i).ChLow + Channel * QRGTbl(i).ChIncr

  GetChannel$ = RTrim$(QRGTbl(i).Prefix) + Format$(Channel, Fmt$) _
     + RTrim$(QRGTbl(i).Postfix)

End Function

Function GetQrgInfo$(TxQRG&)

  Call Qrg2Str(TxQRG&, sTxQrg$, sRxQrg$, sChannel$)

  If Len(sRxQrg$) = 0 Or sRxQrg$ = sTxQrg$ Then
    'nicht in Kanalliste gefunden oder Shift = 0
    m$ = sTxQrg$ & " MHz"
  Else
    m$ = "TX:" & sTxQrg$ & " / RX:" & sRxQrg$ + " MHz"
  End If

  If Len(sChannel$) > 0 Then
    m$ = m$ & " (" & sChannel$ & ")"
  End If

  GetQrgInfo$ = m$

End Function

Function InitQRGTable()

  InitQRGTable = True

  '--- Bereich '[QRG]' laden ---
  ErrNr = GetSection(InParFile$, "[QRG]", True, SectData$, AnzDataLines, Found)
  If ErrNr Then Exit Function

  ReDim QRGTbl(1 To AnzDataLines) As QRGTable

  '--- Parameter einlesen ---
  Do Until Len(SectData$) = 0
    Comment$ = Parse$(SectData$, vbCrLf)
    'Strip off comment
    l$ = Parse$(Comment$, ";")
    If Len(l$) > 0 Then
      i = i + 1
      QRGTbl(i).QRGLow = Val(Parse$(l$, ",")) * 10000&
      QRGTbl(i).QRGHigh = Val(Parse$(l$, ",")) * 10000&
      QRGTbl(i).Shift = Val(Parse$(l$, ",")) * 10000&
      QRGTbl(i).Raster = Val(Parse$(l$, ",")) * 10&

      QRGTbl(i).Prefix = Parse$(l$, ",")
      ChLow$ = Parse$(l$, ",")
      QRGTbl(i).ChLow = Val(ChLow$)
      QRGTbl(i).ChLen = Len(ChLow$)
      QRGTbl(i).ChIncr = Val(Parse$(l$, ","))
      QRGTbl(i).Postfix = Parse$(l$, ",")

      QRGTbl(i).Description = l$
    End If
  Loop

  InitQRGTable = False

End Function

Sub Qrg2Str(linTxQrg&, sTxQrg$, sRxQrg$, sChannel$)

  'linTxQrg - Qrg in kHz
  'output in MHz

  TxQRG& = linTxQrg& / 2.5   'auf 2.5 KHz normieren
  TxQRG& = TxQRG& * 25       'QRG in 100 Hz

  For i = 1 To UBound(QRGTbl)
    If TxQRG& >= QRGTbl(i).QRGLow And TxQRG& <= QRGTbl(i).QRGHigh Then
      Entry = i
      Exit For
    End If
  Next i

  f$ = "#,##0.000#"
  m$ = Format$(CCur(TxQRG&) / 10000@, f$)

  If Entry = 0 Then
    'nicht in Kanalliste gefunden
    sTxQrg$ = Format$(CCur(linTxQrg&) / 1000@, f$)
    sRxQrg$ = ""
    sChannel$ = ""
  Else
    Raster = QRGTbl(Entry).Raster       'Raster in 100 Hz
    TxQRG& = linTxQrg& * 10 / Raster    'auf Raster normieren
    TxQRG& = TxQRG& * Raster            'QRG in 100 Hz
    sTxQrg$ = Format$(CCur(TxQRG&) / 10000@, f$)

    RxQRG& = TxQRG& + QRGTbl(Entry).Shift
    sRxQrg$ = Format$(CCur(RxQRG&) / 10000@, f$)

    If Len(QRGTbl(i).Prefix) > 0 Then
      sChannel$ = GetChannel$(TxQRG&, Entry)
    Else
      sChannel$ = ""
    End If
  End If

End Sub

Function WriteChannelList(FileSpec$)

  ErrNr = OpenFile(FileSpec$, "O", 0, FileNr)
  If ErrNr = 0 Then
    For i = 1 To UBound(QRGTbl)
      'Debug.Print RTrim$(QRGTbl(i).Description)
      If QRGTbl(i).Raster = 0 Then Stop

      m$ = RTrim$(QRGTbl(i).Description)
      ErrNr = WriteLine(FileNr, m$)
      If ErrNr Then Exit For
      ErrNr = WriteLine(FileNr, String$(Len(m$), "-"))
      If ErrNr Then Exit For

      TxQRG& = QRGTbl(i).QRGLow
      Do
        RxQRG& = TxQRG& + QRGTbl(i).Shift
        m$ = Rpad$(GetChannel$(TxQRG&, i), 8)
        m$ = m$ & Rpad$(Format$(RxQRG& / 10000!, "0.000#"), 12)
        If RxQRG& <> TxQRG& Then
          m$ = m$ & Rpad$(Format$(TxQRG& / 10000!, "0.000#"), 12)
        End If
        ErrNr = WriteLine(FileNr, m$)
        If ErrNr Then Exit For
        TxQRG& = TxQRG& + QRGTbl(i).Raster
      Loop Until TxQRG& > QRGTbl(i).QRGHigh

      ErrNr = WriteLine(FileNr, "")
      If ErrNr Then Exit For
    Next i
  End If

  ErrNr = ErrNr Or CloseFile(FileNr)
  WriteChannelList = ErrNr

End Function

